<?php
/**
 * Background Emailer
 *
 * @version 3.0.1
 * @package WooCommerce/Classes
 */

defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'WC_Background_Process', false ) ) {
  include_once dirname( WC_PLUGIN_FILE ) . '/abstracts/class-wc-background-process.php';
}

class WC_Billingo_Plus_Background_Migrator extends WC_Background_Process {

	/**
	 * Initiate new background process.
	 */
	public function __construct() {
		$this->prefix = 'wp_' . get_current_blog_id();
		$this->action = 'wc_billingo_plus_migrate_orders';
		parent::__construct();
	}

	protected function task( $item ) {
    global $wc_billingo_plus;
    if ( ! $item || empty( $item['task'] ) ) {
			return false;
		}

		$process_count = 0;
		$process_limit = 20;

		switch ( $item['task'] ) {
			case 'migrate_orders':
				$process_count = $this->migrate_orders( $process_limit );
				break;
		}

		if ( $process_limit === $process_count ) {
			// Needs to run again.
			return $item;
		} else {
      update_option('_wc_billingo_plus_migrating', false);
      update_option('_wc_billingo_plus_migrated', true);
    }

		return false;
	}

  public function migrate_orders($limit = 20) {
    global $wc_billingo_plus;
    $query = array(
      'limit' => $limit,
			'meta_key'     => '_wc_billingo_plus_migrated',
			'meta_compare' => 'NOT EXISTS'
    );

    $orders = wc_get_orders( $query );
		$count = 0;

		if($orders) {
			foreach ( $orders as $order ) {
        $order_id = $order->get_id();

        $old_meta_keys = array('', '_id', '_pdf', '_dijbekero', '_dijbekero_id', '_dijbekero_pdf', '_own');
        $new_meta_keys = array('invoice_name', 'invoice_id', 'invoice_pdf', 'proform_name', 'proform_id', 'proform_pdf', 'own');

        foreach ($old_meta_keys as $index => $old_meta_key) {
          $old_meta_value = $order->get_meta('_wc_billingo'.$old_meta_key);
          if($old_meta_value) {
            $order->update_meta_data('_wc_billingo_plus_'.$new_meta_keys[$index], $old_meta_value);
          }
        }

        if($wc_billingo_plus->check_payment_method_options($order->get_payment_method(), 'complete')) {
          $order->update_meta_data('_wc_billingo_plus_completed', true);
        }

        $order->update_meta_data('_wc_billingo_plus_migrated', true);

        $order-save();

				$count++;
			}
		}

		return $count;
  }

}
