<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'WC_Billingo_Plus_Health_Check', false ) ) :

	class WC_Billingo_Plus_Health_Check {

		//Init notices
		public static function init() {

      add_filter( 'debug_information', array( __CLASS__, 'debug_info' ) );
    	add_filter( 'site_status_tests', array(__CLASS__, 'status_tests') );
    	add_action( 'wp_ajax_health-check-wc-billingo_plus_test', array(__CLASS__, 'status_tests_ajax') );
		}

		public static function debug_info($debug) {
      $billingo = array(
  			'wc_billingo_plus' => array(
  				'label' => esc_html__( 'WooCommerce Billingo Plus', 'wc-billingo-plus' ),
  				'description' => sprintf(
  					esc_html__(
  						'Diagnosztikai információk a Billingo integrációhoz kapcsolódóan. Ha kérdésed van vagy valami nem működik, mellékeld ezeket az adatokat is: %1$s',
  						'wc-billingo-plus'
  					),
						'<a href="https://szamlazz.visztpeter.me/" target="_blank" rel="noopener noreferrer">'.esc_html__('Támogatás', 'wc-billingo-plus').'</a>'
  				),
  				'fields' => self::debug_info_data(),
  			),
  		);
  		$debug = array_merge($debug, $billingo);
  		return $debug;
		}

    public static function status_tests($core_tests) {

      $core_tests['direct']['wc_billingo_plus_test'] = array(
        'label' => esc_html__( 'Billingo Plus követelmények', 'wc-billingo-plus' ),
        'test'  => function() {
          $settings = get_option( 'woocommerce_wc_billingo_plus_settings', null );

          $result = array(
      			'label'       => esc_html__( 'Billingo Plus követelmények', 'wc-billingo-plus' ),
      			'status'      => 'good',
      			'badge'       => array(
      				'label' => esc_html__( 'Billingo Plus', 'wc-billingo-plus' ),
      				'color' => 'blue',
      			),
      			'description' => esc_html__( 'A weboldal és a tárhely minden követelményenek megfelel egy sikeres számlakészítéshez a WooCommerce Billingo Plus bővítménnyel.', 'wc-billingo-plus' ),
      		);

          //Username/password
          if($settings['public_key'] && $settings['private_key']) {
            //all good
          } else {
            $result['status'] = 'critical';
            $result['badge']['color'] = 'red';
            $result['description'] = esc_html__('A WooCommerce Billingo Plus bővítmény használatához add meg a bővítmény beállításaiban a publikus és privát kulcsokat.', 'wc-billingo-plus');
            $result['actions'] = sprintf(
      				'<p><a href="%s" target="_blank" rel="noopener noreferrer">%s <span aria-hidden="true" class="dashicons dashicons-admin-generic"></span></a></p>',
      				esc_url( admin_url( 'admin.php?page=wc-settings&tab=integration&section=wc_billingo_plus' ) ),
      				esc_html__( 'Bellítások', 'wc-billingo-plus' )
      			);
          }

					if(!$settings['block_uid'] || $settings['block_uid'] == '') {
						$result['status'] = 'critical';
						$result['badge']['color'] = 'red';
						$result['description'] = __('A <strong>WooCommerce Billingo Plus</strong> bővítmény használatához ki kell választanod egy számaltömböt a beállításokban.', 'wc-billingo-plus');
						$result['actions'] = sprintf(
							'<p><a href="%s" target="_blank" rel="noopener noreferrer">%s <span aria-hidden="true" class="dashicons dashicons-admin-generic"></span></a></p>',
							esc_url( admin_url( 'admin.php?page=wc-settings&tab=integration&section=wc_billingo_plus' ) ),
							esc_html__( 'Bellítások', 'wc-billingo-plus' )
						);
					}

          return $result;
        }
      );

			//Debug mode is turned on
			$core_tests['direct']['wc_billingo_plus_debug'] = array(
				'label' => esc_html__( 'Billingo Plus fejlesztői mód', 'wc-billingo-plus' ),
				'test'  => function() {
					$settings = get_option( 'woocommerce_wc_billingo_plus_settings', null );

					$result = array(
						'label'       => esc_html__('Billingo Plus fejlesztői mód ki van kapcsolva', 'wc-billingo-plus'),
						'status'      => 'good',
						'badge'       => array(
							'label' => esc_html__( 'Billingo Plus', 'wc-billingo-plus' ),
							'color' => 'blue',
						),
						'description' => esc_html__( 'A WooCommerce Billingo Plus fejlesztői módja ki van kapcsolva.', 'wc-billingo-plus' ),
						'test'        => 'wc_billingo_plus_check_debug_mode',
					);

					//If debug mode is turned on
					if($settings['debug'] && $settings['debug'] == 'yes') {
						$result['label'] = esc_html__('Billingo Plus fejlesztői mód be van kapcsolva', 'wc-billingo-plus');
						$result['status'] = 'critical';
						$result['badge']['color'] = 'red';
						$result['description'] = esc_html__('A WooCommerce Billingo Plus bővítmény fejlesztői módja be van kapcsolva. Éles használat esetén ezt mindenképp kapcsold ki, amit megtehetsz a bővítmény beállításaiban.', 'wc-billingo-plus');
						$result['actions'] = sprintf(
							'<p><a href="%s" target="_blank" rel="noopener noreferrer">%s <span aria-hidden="true" class="dashicons dashicons-admin-generic"></span></a></p>',
							esc_url( admin_url( 'admin.php?page=wc-settings&tab=integration&section=wc_billingo_plus' ) ),
							esc_html__( 'Bellítások', 'wc-billingo-plus' )
						);
					}

					return $result;
				}
			);

      $core_tests['async']['wc_billingo_plus_generate'] = array(
        'label' => esc_html__( 'Billingo Plus számlakészítés', 'wc-billingo-plus' ),
        'test'  => 'wc_billingo_plus_test',
      );

      return $core_tests;
  	}

    public static function status_tests_ajax() {
      check_ajax_referer( 'health-check-site-status' );

			$result = array(
        'label'       => esc_html__( 'Billingo Plus összekapcsolás sikeres.', 'wc-billingo-plus' ),
        'status'      => 'good',
        'badge'       => array(
					'label' => esc_html__( 'Billingo Plus', 'wc-billingo-plus' ),
          'color' => 'blue',
        ),
        'description' => sprintf(
          '<p>%s</p>',
          esc_html__( "A WooCommerce Billingo Plus bővítménynek sikerült kapcsolatot létesítenie a megadott api kulcsokkal. Úgy néz ki, minden rendben.", 'wc-billingo-plus' )
        ),
        'actions'     => '',
        'test'        => 'wc_billingo_plus_test_generate_invoice',
      );

			global $wc_billingo_plus;
			$billingo = $wc_billingo_plus->get_billingo_api();
			try {
				$billingo_vat_ids = $billingo->get('vat');
			} catch (Exception $e) {
				$error_message = $e->getMessage();

				$result['label'] = __('Billingo összekapcsolás sikertelen', 'wc-billingo-plus');
				$result['status'] = 'critical';
				$result['badge']['color'] = 'red';
				$result['description'] = sprintf(
					'<p>%s</p><p>%s</p>',
					esc_html__( "A WooCommerce Billingo Plus bővítménynek nem sikerült kommunikálnia a Billingo szerverével. A pontos hibaüzenet ez volt:", 'wc-billingo-plus' ),
					esc_html($e->getMessage())
				);

				if (strpos($error_message, 'User not found!') !== false) {

					$result['description'] = sprintf(
						'<p>%s</p><p>%s</p>',
						esc_html__( "A WooCommerce Billingo Plus bővítménynek nem sikerült kommunikálnia a Billingo szerverével. Ellenőrizd, hogy a privát és a publikus kulcs egyezik e a Billingo API menüben mutatott kulcsokkal. A pontos hibaüzenet ez volt:", 'wc-billingo-plus' ),
						esc_html($e->getMessage())
					);
				}

				if (strpos($error_message, 'Cannot handle token') !== false) {

					$result['description'] = sprintf(
						'<p>%s</p><p>%s</p>',
						esc_html__( "A WooCommerce Billingo Plus bővítménynek nem sikerült kommunikálnia a Billingo szerverével. Ellenőrizd, hogy a szervereden lévő pontos idő azonos e a Billingo szerverén lévő idővel. A pontos hibaüzenet ez volt:", 'wc-billingo-plus' ),
						esc_html($e->getMessage())
					);
				}

				if (strpos($error_message, 'User needs to be premium') !== false) {

					$result['description'] = sprintf(
						'<p>%s</p><p>%s</p>',
						esc_html__( "A WooCommerce Billingo Plus bővítménynek nem sikerült kommunikálnia a Billingo szerverével. Úgy néz ki az ingyenes verziót használod, melyben nem elérhető az API használat. API csak a 30 napos próbaidőszakban vagy a Prémium verzióban elérhető. A pontos hibaüzenet ez volt:", 'wc-billingo-plus' ),
						esc_html($e->getMessage())
					);
				}
			}

      wp_send_json_success($result);
  	}

    public static function debug_info_data() {
      $debug_info = array();

			//PRO verzió
			$debug_info['wc_billingo_plus_pro_version'] = array(
				'label'   => esc_html__('PRO verzió', 'wc-billingo-plus'),
				'value'   => intval(get_option('_wc_billingo_pro_enabled'))
			);

			//Invoice path
			$UploadDir = wp_upload_dir();
			$UploadURL = $UploadDir['basedir'];
			$location  = realpath($UploadURL . "/wc_billingo_plus/");
			$debug_info['wc_billingo_plus_path'] = array(
				'label'   => esc_html__('Útvonal', 'wc-billingo-plus'),
				'value'   => $location
			);

			//IPN URL
			$settings_api = new WC_Billingo_Plus_Settings();

			//Payment options
			$payment_options = get_option('wc_billingo_plus_payment_method_options');
			$debug_info['wc_billingo_plus_payment_options'] = array(
				'label'   => esc_html__('Fizetési módok', 'wc-billingo-plus'),
				'value'   => print_r($payment_options, true)
			);

			//Rounding options
			$rounding_options = get_option('wc_billingo_plus_rounding_options');
			$debug_info['wc_billingo_plus_rounding_options'] = array(
				'label'   => esc_html__('Kerekítés', 'wc-billingo-plus'),
				'value'   => print_r($rounding_options, true)
			);

			//Display saved settings
      $settings = get_option( 'woocommerce_wc_billingo_plus_settings', null );
      $options = $settings_api->form_fields;
			unset($options['private_key']);

      foreach ($options as $option_id => $option) {
				if(!in_array($option['type'], array('pro', 'title', 'payment_methods', 'rounding'))) {
					$debug_info[$option_id] = array(
						'label'   => esc_html($option['title']),
						'value'   => esc_html($settings[$option_id])
					);
				}
      }

      return $debug_info;
    }

  }

	WC_Billingo_Plus_Health_Check::init();

endif;
