<?php

if ( ! class_exists( 'WC_Billingo_Plus_Settings' ) ) :

if ( ! class_exists( 'WC_Billingo_Plus_Background_Migrator', false ) ) {
	include_once dirname( __FILE__ ) . '/class-background-migrate.php';
}

class WC_Billingo_Plus_Settings extends WC_Integration {
	public static $activation_url;
	protected static $background_migrator;

	/**
	 * Init and hook in the integration.
	 */
	public function __construct() {
		$this->id                 = 'wc_billingo_plus';
		$this->method_title       = __( 'Billingo Plus', 'wc-billingo-plus' );
		$this->method_description = __( 'Nézd át az alábbi beállításokat ahhoz, hogy számlákat tudj generálni. Fontos megjegyezni, hogy ez nem egy hivatalos Billingo bővítmény. Ha valami hibát tapasztalsz, ne a Billingo supportnál, hanem nálam, a bővítmény készítőjénél érdeklődj: info@visztpeter.me.', 'wc-billingo-plus' );

		//Setup migration utility
		if ( ! self::$background_migrator ) {
			self::$background_migrator = new WC_Billingo_Plus_Background_Migrator();
		}

		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();

		// Action to save the fields
		add_action( 'woocommerce_update_options_integration_' .  $this->id, array( $this, 'process_admin_options' ) );
		add_action( 'woocommerce_update_options_integration_' .  $this->id, array( $this, 'save_payment_options' ) );

		//Check and save PRO version
		add_action( 'wp_ajax_wc_billingo_pro_check', array( $this, 'pro_check' ) );
		add_action( 'wp_ajax_wc_billingo_pro_deactivate', array( $this, 'pro_deactivate' ) );
		add_action( 'wp_ajax_wc_billingo_plus_reload_blocks', array( $this, 'reload_blocks' ) );
		add_action( 'wp_ajax_wc_billingo_plus_migrate', array( $this, 'migrate_settings' ) );

		//Define activation url
		self::$activation_url = 'https://szamlazz.visztpeter.me/';

	}

	public function migrate_settings() {
		check_ajax_referer( 'wc-billingo-plus-migrate', 'security' );

		if ( ! current_user_can( 'manage_woocommerce' ) ) {
			wp_die( esc_html__( 'Cheatin&#8217; huh?' ) );
		}

		//Check if already migrated
		if(get_option('_wc_billingo_plus_migrated')) {
			return false;
		}

		//Gather data
		$option_pairs = array(
			'public_key' => 'public_key',
			'secret_key' => 'private_key',
			'invoice_block' => 'block_uid',
			'note' => 'note',
			'invoice_lang' => 'language',
			'email' => 'auto_email'
		);

		foreach ($option_pairs as $old_option_id => $new_option_id) {
			$old_option_value = get_option('wc_billingo_'.$old_option_id);
			$this->update_option($new_option_id, $old_option_value);
		}

		if(get_option('wc_billingo_electronic') == 'yes') {
			$this->update_option('invoice_type', 'electronic');
		}

		//Save payment options
		$accounts = array();
		$payment_methods = $this->get_payment_methods();
		foreach ($payment_methods as $payment_method_id => $payment_method) {
			$billingo_id = (int)get_option('wc_billingo_payment_method_' . $payment_method_id);
			$deadline = (int)get_option('wc_billingo_paymentdue_' . $payment_method_id);
			$complete = (int)get_option('wc_billingo_mark_as_paid_' . $payment_method_id);
			$proform =	(int)get_option('wc_billingo_proforma_' . $payment_method_id);

			if($billingo_id) {
				$accounts[$payment_method_id] = array(
					'billingo_id' => $billingo_id,
					'deadline' => $deadline,
					'complete' => ($complete) ? true : false,
					'proform'  => ($proform) ? true : false
				);
			}
		}
		update_option( 'wc_billingo_plus_payment_method_options', $accounts );

		//Migrate orders too. This might take a while
		self::$background_migrator->push_to_queue( array( 'task' => 'migrate_orders' ) );
		self::$background_migrator->save()->dispatch();
		update_option('_wc_billingo_plus_migrating', true);

		//Disable original plugin
		deactivate_plugins('/billingo/index.php');

		wp_die();
	}

	/**
	 * Initialize integration settings form fields.
	 *
	 * @return void
	 */
	public function init_form_fields() {
		$pro_required = false;
		$pro_icon = false;
		if(!get_option('_wc_billingo_pro_enabled')) {
			$pro_required = true;
			$pro_icon = '<i class="wc_billingo_pro_label">PRO</i>';
		}

		$settings_top = array(
			'pro_key' => array(
				'title'    => __( 'PRO verzió', 'wc-billingo-plus' ),
				'type'     => 'pro'
			),
			'section_auth' => array(
				'title' => __( 'Authentikáció', 'wc-billingo-plus' ),
				'type'  => 'title',
				'description'  => __( 'Add meg a Billingo-n az API menüben generált nyilvános és privát kulcsokat.', 'wc-billingo-plus' ),
			),
			'public_key' => array(
				'title'    => __( 'Nyilvános kulcs', 'wc-billingo-plus' ),
				'type'     => 'text',
			),
			'private_key' => array(
				'title'    => __( 'Privát kulcs', 'wc-billingo-plus' ),
				'type'     => 'text',
				'description'		 => __("A kulcs sima szövegként van tárolva az adatbázisban. Javasolt, hogy inkább a WordPress konfigurációs fájlába add meg, úgy biztonságosabb. Hagyd üresen ezt a mezőt és a wp-config.php fájlhoz add hozzá ezt a sort: <pre>define( 'WC_BILLINGO_PRIVATE_KEY', 'kulcs' );</pre>", 'wc-billingo-plus' )
			),
		);

		$settings_rest = array(
			'invoice_settings' => array(
				'title' => __( 'Számla beállítások', 'wc-billingo-plus' ),
				'type'  => 'title',
				'description'  => __( 'A számlával kapcsolatos alapértelmezett beállítások.', 'wc-billingo-plus' ),
			),
			'block_uid' => array(
				'title'    => __( 'Számlatömb', 'wc-billingo-plus' ),
				'type'		 => 'select',
				'class'    => 'chosen_select test',
				'options'  => $this->get_invoice_blocks(),
				'description'		 => $this->get_fetch_error_message('blocks').__("Új számlatömböt a Billingo weboldalán a Beállítások / Számlatömbök menüpontban tudsz létrehozni. Ha nem látod a legördülőben a létrehozott számlatömböt, nyomd meg a frissítés gombot.", 'wc-billingo-plus' )
			),
			'invoice_type' => array(
        'title'    => __( 'Számla típusa', 'wc-billingo-plus' ),
  			'class'    => 'chosen_select',
  			'css'      => 'min-width:300px;',
  			'type'     => 'select',
  			'options'     => array(
  				'electronic'  => __( 'Elektronikus', 'wc-billingo-plus' ),
  				'paper' => __( 'Papír', 'wc-billingo-plus' )
  			)
			),
			'invoice_type_company' => array(
        'title'    => __( 'Számla típusa céges rendelésnél', 'wc-billingo-plus' ),
  			'class'    => 'chosen_select',
  			'css'      => 'min-width:300px;',
  			'type'     => 'select',
  			'options'     => array(
					'' => __( 'Alapértelmezett', 'wc-billingo-plus' ),
  				'electronic'  => __( 'Elektronikus', 'wc-billingo-plus' ),
  				'paper' => __( 'Papír', 'wc-billingo-plus' )
  			),
				'desc_tip' => __( 'Céges rendelés esetén a számla típusa. A rendelés akkor céges, ha volt megadva cégnév.')
			),
			'payment_deadline' => array(
        'title'    => __( 'Fizetési határidő(nap)', 'wc-billingo-plus' ),
  			'type'     => 'number'
			),
			'note' => array(
        'title'    => __( 'Megjegyzés', 'wc-billingo-plus' ),
  			'type'     => 'textarea',
				'desc_tip' => __( 'A {customer_email} kóddal megjelenítheted a vásárló email címét a megjegyzésben, a telefonszámát pedig a {customer_phone} kóddal.')
			),
			'afakulcs' => array(
				'type'		 => 'select',
				'class'    => 'chosen_select',
				'title'    => __( 'Áfakulcs', 'wc-billingo-plus' ),
				'options'  => $this->get_vat_options(),
				'description' => $this->get_fetch_error_message().__( 'Az áfakulcs, ami a számlán szerepelni fog. Alapértelmezetten a WooCommerce Adó menüpontban megadott adózási osztályokhoz tartozó "Adó Neve" mező értékét használja adókulcsnak, akkor, ha ezek valamelyike van megadva: AM, EU, FAD, ÁTHK, AAM, AKK, TAM, EUK, MAA. Ha nem ezek közül van megadva, akkor a százalékos értéket fogja kiszámolni a nettó/bruttó tétel árakból.', 'wc-billingo-plus' ),
				'default'  => ''
			),
			'language' => array(
				'type'		 => 'select',
				'class'    => 'chosen_select',
				'title'    => __( 'Számla nyelve', 'wc-billingo-plus' ),
				'options'  => array(
					'hu' => __( 'Magyar', 'wc-billingo-plus' ),
					'de' => __( 'Német', 'wc-billingo-plus' ),
					'en' => __( 'Angol', 'wc-billingo-plus' ),
					'it' => __( 'Olasz', 'wc-billingo-plus' ),
					'fr' => __( 'Francia', 'wc-billingo-plus' ),
					'hr' => __( 'Horvát', 'wc-billingo-plus' ),
					'ro' => __( 'Román', 'wc-billingo-plus' ),
					'sk' => __( 'Szlovák', 'wc-billingo-plus' )
				),
				'default'  => 'hu'
			),
			'language_wpml' => array(
				'title'    => __( 'WPML és Polylang kompatibilitás', 'wc-billingo-plus' ),
				'type'     => 'checkbox',
				'desc_tip' => __('Ha be van kapcsolva, akkor a WPML vagy a Polylang által a rendelés adataiban tárolt nyelvkódot fogja használja a számla létrehozásához', 'wc-billingo-plus')
			),
			'unit_type' => array(
				'title'    => __( 'Mennyiségi egység', 'wc-billingo-plus' ),
				'type'     => 'text',
				'desc_tip' => __('Ez lesz az alapértelmezett mennyiségi egység a számlán a tételeknél, például "db". A termék szerkesztésekor a Haladó fülön megadható egyedi mennyiségi egység külön-külön minden termékhez.', 'wc-billingo-plus')
			),
			'company_name' => array(
				'title'    => __( 'Cégnév + név', 'wc-billingo-plus' ),
				'type'     => 'checkbox',
				'desc_tip' => __('Ha be van kapcsolva és vásárláskor cégnév van megadva, a vásárló nevét is feltünteti egy kötőjellel a cégnév után a számlán.', 'wc-billingo-plus')
			),
			'rounding' => array(
				'title'    => __( 'Kerekítés', 'wc-billingo-plus' ),
				'type'     => 'rounding'
			),
			'section_automatic' => array(
				'title' => __( 'Automatizálás', 'wc-billingo-plus' ).$pro_icon,
				'type'  => 'title',
				'description'  => __( 'Számlakészítés automatizálásával kapcsolatos beállítások. Ha az adott fizetési módnál a teljesítettnek jelölés be van pipálva, akkor a rendelés teljesítettnek jelölésekor automatikusan kifizetettnek jelöli meg a számlát. Ha a díjbekérő be van pipálva, akkor a rendelés létrehozásakor díjbekérőt is készít. A határidő minden fizetési módhoz egyedileg beállítható(az alapértelmezett érték a fenti beállításokban van).', 'wc-billingo-plus' ),
			),
			'auto_generate' => array(
				'disabled' => $pro_required,
        'title'    => __( 'Automata számlakészítés', 'wc-billingo-plus' ),
  			'type'     => 'checkbox',
  			'desc_tip'     => __( 'Ha be van kapcsolva, akkor a rendelés lezárásakor automatán kiállításra kerül a számla és a Billingo elküldi a vásárló email címére.', 'wc-billingo-plus' ),
			),
			'auto_invoice_status' => array(
				'type' => 'select',
				'title' => __( 'Számlakészítés státusznál', 'wc-billingo-plus' ),
				'class' => 'wc-enhanced-select',
				'default' => 'wc-completed',
				'options'  => wc_get_order_statuses(),
				'desc_tip' => __( 'Ha a rendelés ebbe a státuszba kerül, akkor fogja automatán megcsináni a számlát(csak ha az automata számlakészítés be van kapcsolva). ', 'wc-billingo-plus' ),
			),
			'auto_void_status' => array(
				'type' => 'select',
				'title' => __( 'Automata sztornózás', 'wc-billingo-plus' ),
				'class' => 'wc-enhanced-select',
				'default' => 'no',
				'options'  => $this->get_order_statuses_for_void(),
				'desc_tip' => __( 'Ha a rendelés ebbe a státuszba kerül, akkor automatán létrehozza a sztornó számlát(természetesen akkor, ha már van a rendeléshez számla csinálva és ha az automata számlakészítés be van kapcsolva). ', 'wc-billingo-plus' ),
			),
			'payment_methods' => array(
				'title'    => __( 'Fizetési módok', 'wc-billingo-plus' ),
				'type'     => 'payment_methods',
				'disabled' => $pro_required,
			),
			'section_coupons' => array(
				'title' => __( 'Kuponok', 'wc-billingo-plus' ),
				'type'  => 'title',
				'description'  => __( 'A kuponos rendelések számlán történő megjelenítésével kapcsolatos beállítások', 'wc-billingo-plus' ),
			),
			'separate_coupon' => array(
				'title'    => __( 'Kupon külön tételként', 'wc-billingo-plus' ),
				'type'     => 'checkbox',
				'class'		 => 'wc-billingo-plus-toggle-group-coupon',
				'desc_tip' => __('Ha be van kapcsolva, akkor a rendeléshez használt kupont külön tételként tünteti fel a számlán, a vásárolt tételeknél pedig az eredeti árat fogja mutatni, nem a kedvezménnyel levontat.', 'wc-billingo-plus')
			),
			'separate_coupon_name' => array(
				'title'    => __( 'Kupon tétel neve', 'wc-billingo-plus' ),
				'type'     => 'text',
				'class'		 => 'wc-billingo-plus-toggle-group-coupon-item',
				'placeholder' => 'Kedvezmény',
				'desc_tip' => __('Rendeléshez használt kupont külön tételként tünteti fel a számlán, ennek a nevét adhatod itt meg. Alapértelmezett: Kedvezmény', 'wc-billingo-plus')
			),
			'separate_coupon_desc' => array(
				'title'    => __( 'Kupon tétel leírása', 'wc-billingo-plus' ),
				'type'     => 'textarea',
				'class'		 => 'wc-billingo-plus-toggle-group-coupon-item',
				'placeholder' => '{kedvezmeny_merteke} kedvezmény a következő kupon kóddal: {kupon}',
				'desc_tip' => __('Rendeléshez használt kupont külön tételként tünteti fel a számlán, ennek a tételnek a leírását adhatod meg itt. Alapértelmezett: {kedvezmeny_merteke} kedvezmény a következő kupon kóddal: {kupon}', 'wc-billingo-plus')
			),
			'discount_note' => array(
				'title'    => __( 'Akciós termék megjegyzés', 'wc-billingo-plus' ),
				'type'     => 'textarea',
				'desc_tip' => __('A tétel megjegyzésében fel tudod tüntetni az eredeti árat és a kedvezmény mértékét. Használd ezeket a helyettesítőket: {eredeti_ar}, {kedvezmeny_merteke}, {kedvezmenyes_ar}', 'wc-billingo-plus')
			),
			'hide_free_shipping' => array(
				'title'    => __( 'Ingyenes szállítás elrejtése', 'wc-billingo-plus' ),
				'type'     => 'checkbox',
				'desc_tip' => __('Ha be van kapcsolva, akkor nem tüntetni fel az ingyenes szállításra vonatkozó tételt a számlán.', 'wc-billingo-plus')
			),
			'section_vat_number' => array(
				'title' => __( 'Adószám', 'wc-billingo-plus' ),
				'type'  => 'title',
				'description'  => __( '100.000 Ft áfatartalomnál magasabb rendeléseknél kötelező adószámot bekérni, ha van a vevőnek. Az alábbi opciókkal egy adószám mezőt lehet hozzáadni a pénztár űrlaphoz.', 'wc-billingo-plus' ),
			),
			'vat_number_form' => array(
        'title'    => __( 'Adószám mező vásárláskor', 'wc-billingo-plus' ),
  			'type'     => 'checkbox',
				'class'		 => 'wc-billingo-plus-toggle-group-vatnumber',
				'description' => $this->get_vat_number_error_message(),
  			'desc_tip'     => __( 'A számlázási adatok alján egy új mezőben lesz bekérve. Eltárolja a rendelés adataiban, illetve számlára is ráírja. Ha kézzel kell megadni utólag a rendeléskezelőben, akkor az egyedi mezőknél az "adoszam" mezőt kell kitölteni.', 'wc-billingo-plus' ),
			),
			'vat_number_validate' => array(
				'title'    => __( 'Adószám ellenőrzése', 'wc-billingo-plus' ),
				'type'     => 'checkbox',
				'class'		 => 'wc-billingo-plus-toggle-group-vatnumber-item',
				'desc_tip'     => __( 'Az adószám formátumának ellenőrzése', 'wc-billingo-plus' ),
			),
			'vat_number_form_min' => array(
        'title'    => __( 'Csak 100.000 Ft áfa felett', 'wc-billingo-plus' ),
  			'type'     => 'checkbox',
				'class'		 => 'wc-billingo-plus-toggle-group-vatnumber-item',
  			'desc_tip'     => __( 'Az adószám mező csak akkor látszik, ha 100.000 Ft felett van az áfatartalom.', 'wc-billingo-plus' ),
			),
			'vat_number_notice' => array(
				'title'    => __( 'Adószám figyelmeztetés', 'wc-billingo-plus' ),
				'type'     => 'textarea',
				'class'		 => 'wc-billingo-plus-toggle-group-vatnumber-item',
				'default'	 => __( 'A vásárlás áfatartalma több, mint 100.000 Ft, ezért amennyiben rendelkezik adószámmal, azt kötelező megadni a számlázási adatoknál.', 'wc-billingo-plus'),
				'desc_tip'     => __( 'Ez az üzenet jelenik meg, ha az adószám mező be van pipálva felül a fizetés oldalon és 100.000 Ft felett van az áfatartalom.', 'wc-billingo-plus' ),
			),
			'section_emails' => array(
				'title' => __( 'Számla megosztás', 'wc-billingo-plus' ),
				'type'  => 'title',
				'description'  => __( 'Beállítások a számlák vásárlónak történő kiküldésével és letöltésével kapcsolatban.', 'wc-billingo-plus' ),
			),
			'auto_email' => array(
				'title'    => __( 'Számlaértesítő', 'wc-billingo-plus' ),
				'type'     => 'checkbox',
				'desc_tip'     => __( 'Ha be van kapcsolva, akkor a vásárlónak a billingo kiküldi a számlaértesítőt automatán.', 'wc-billingo-plus' ),
				'default'  => 'yes'
			),
			'download_invoice' => array(
				'title'    => __( 'Számlák letöltése', 'wc-billingo-plus' ).$pro_icon,
				'type'     => 'checkbox',
				'description'     => __( 'Ha be van kapcsolva, akkor a PDF fájlt letölti a webáruházadba az uploads/wc_billingo_plus mappába. Ha nincs bekapcsolva, akkor csak a Billingo oldalára mutató letöltés linket tárolja el. Fontos, hogy 1 perc alatt maximum 10 számlát lehet letölteni. Tehát ha a munkafolyamatodban egyszerre sok rendelést teljesítész, ne használd ezt az opciót.', 'wc-billingo-plus' ),
				'default'  => 'no'
			),
			'email_attachment' => array(
				'title'    => __( 'Számlák beillesztése az e-mailekbe', 'wc-billingo-plus' ).$pro_icon,
				'type'     => 'checkbox',
				'disabled' => $pro_required,
				'class'		 => 'wc-billingo-plus-toggle-group-emails',
				'desc_tip' => __( 'A számlák a WooCommerce által kiküldött e-mailekbe lesznek elhelyezve, link formájában. Érdemes ilyenkor a számlaértesítőt kikapcsolni. Kiválaszthatod, hogy melyik e-mailhez csatolja a számlát. Egyszerre többet is kiválaszthatsz.', 'wc-billingo-plus' ),
			),
			'email_attachment_file' => array(
				'title'    => __( 'Számlák csatolása az emailekhez', 'wc-billingo-plus' ).$pro_icon,
				'type'     => 'checkbox',
				'disabled' => $pro_required,
				'class'		 => 'wc-billingo-plus-toggle-group-emails',
				'desc_tip' => __( 'Ha a számlák letöltése opció aktív, akkor a PDF fájlokat csatolni tudja a WooCommerce emailekhez.', 'wc-billingo-plus' ),
			),
			'email_attachment_invoice' => array(
				'type' => 'multiselect',
				'title' => __( 'Végszámla csatolása', 'wc-billingo-plus' ),
				'class' => 'wc-enhanced-select wc-billingo-plus-toggle-group-emails-item',
				'default' => array('customer_completed_order', 'customer_invoice'),
				'options'  => $this->get_email_types(),
			),
			'email_attachment_proform' => array(
				'type' => 'multiselect',
				'title' => __( 'Díjbekérő számla csatolása', 'wc-billingo-plus' ),
				'class' => 'wc-enhanced-select wc-billingo-plus-toggle-group-emails-item',
				'default' => array('customer_processing_order', 'customer_on_hold_order'),
				'options'  => $this->get_email_types(),
			),
			'email_attachment_void' => array(
				'type' => 'multiselect',
				'title' => __( 'Sztornó számla csatolása', 'wc-billingo-plus' ),
				'class' => 'wc-enhanced-select wc-billingo-plus-toggle-group-emails-item',
				'default' => array('customer_refunded_order', 'cancelled_order'),
				'options'  => $this->get_email_types(),
			),
			'email_attachment_position' => array(
				'type' => 'select',
				'class' => 'wc-enhanced-select wc-billingo-plus-toggle-group-emails-item',
				'desc_tip' => __( 'Hol legyenek a számla letöltésre vonatkozó instrukciók beágyazva az e-mailekbe?', 'wc-billingo-plus' ),
				'title' => __( 'E-mail link pozíció', 'wc-billingo-plus' ),
				'default' => 'beginning',
				'options'  => array(
					'beginning' => __( 'Az e-mail elején', 'woocommerce' ),
					'end'  => __( 'Az e-mail végén', 'woocommerce' ),
				),
			),
			'customer_download' => array(
				'title'    => __( 'Számlák a profilban', 'wc-billingo-plus' ),
				'type'     => 'checkbox',
				'desc_tip'     => __( 'Ha be van kapcsolva, akkor a díjbekérőt és a számlát is le tudja tölteni a felhasználó belépés után, a Rendeléseim menüben.', 'wc-billingo-plus' ),
				'default'  => 'no'
			),
			'section_other' => array(
				'title' => __( 'Egyéb beállítások', 'wc-billingo-plus' ),
				'type'  => 'title'
			),
			'nev_csere' => array(
				'title'    => __( 'Vezetéknév / keresztnév csere', 'wc-billingo-plus' ),
				'type'     => 'checkbox',
				'desc_tip'     => __( 'Ha rosszul jelenik meg a számlán a vevő neve, ezzel az opcióval meg lehet cserélni a vezetéknevet és a keresztnevet.', 'wc-billingo-plus' ),
			),
			'error_email' => array(
				'title'    => __( 'Email cím hibajelzéshez', 'wc-billingo-plus' ),
				'type'     => 'text',
				'default'  => get_option('admin_email'),
				'desc_tip'     => __( 'Ha megadsz egy email címet, akkor ide küld a rendszer egy levelet, ha volt a webáruházban egy sikertelen automata számlakészítés. Hagyd üresen, ha nem szeretnél emailt kapni(a számlázz.hu is szokott küldeni).', 'wc-billingo-plus' ),
			),
			'defer' => array(
        'title'    => __( 'Késleltetett számlakészítés', 'wc-billingo-plus' ),
  			'type'     => 'checkbox',
  			'desc_tip' => __( 'Ha be van kapcsolva, akkor a számlát nem a vásárlás közben, hanem után készíti el automatikusan, így a vásárló sokkal gyorsabban jut el a "Köszönöm" oldalra. Fontos, hogy ebben az esetben nem lehet a számlát a WooCommerce levelekhez csatolni, mert azokat előbb kiküldi a vásárlónak, minthogy a számla elkészült volna, tehát használd a Számlázz.hu féle számlaértesítőt.', 'wc-billingo-plus' ),
			),
			'debug' => array(
				'title'    => __( 'Fejlesztői mód', 'wc-billingo-plus' ),
				'type'     => 'checkbox',
				'desc_tip' => __( 'Ha be van kapcsolva, akkor a Billingo-nak küldött adatokat naplózni fogja a bővítmény, amit a WooCommerce / Állapot / Naplók menüpontban találsz.', 'wc-billingo-plus' ),
			),
			'uninstall' => array(
				'title'    => __( 'Beállítások törlése', 'wc-billingo-plus' ),
				'type'     => 'checkbox',
				'desc_tip' => __( 'Ha be van kapcsolva, akkor a bővítmény törlése esetén a tárolt beállításokat is le fogja törölni az adatbázisból.', 'wc-billingo-plus' ),
			)
		);

		$load_settings = false;
		if(isset($_POST['woocommerce_wc_billingo_plus_public_key'])) {
			if($_POST['woocommerce_wc_billingo_plus_public_key'] != '') {
				$load_settings = true;
			}
		} else {
			if($this->get_option('public_key')) {
				$load_settings = true;
			}
		}

		if($load_settings) {
			$this->form_fields = array_merge($settings_top, $settings_rest);
		} else {
			$this->form_fields = array_merge($settings_top);
		}
	}

	//Order statuses
	public function get_order_statuses_for_void() {
		$built_in_statuses = array("no"=>__("Kikapcsolva")) + wc_get_order_statuses();
		return $built_in_statuses;
	}

	//Get invoice blocks
	public static function get_invoice_blocks() {
		global $wc_billingo_plus;
		$blocks = $wc_billingo_plus->get_billingo_invoice_blocks();
		return $blocks;
	}

	//Get VAT IDs
	public static function get_vat_options() {
		global $wc_billingo_plus;
		$vat_ids = $wc_billingo_plus->get_billingo_vat_ids(true);
		return array("" => "Alapértelmezett") + $vat_ids;
	}

  //Get payment methods
  public static function get_payment_methods() {
    $available_gateways = WC()->payment_gateways->payment_gateways();
		$payment_methods = array();
		foreach ($available_gateways as $available_gateway) {
			if($available_gateway->enabled == 'yes') {
				$payment_methods[$available_gateway->id] = $available_gateway->title;
			}
		}
    return $payment_methods;
  }

	//Get email ids
	public static function get_email_types() {
		$mailer = WC()->mailer();
		$email_templates = $mailer->get_emails();
		$emails = array();
		$disabled = ['failed_order', 'customer_note', 'customer_reset_password', 'customer_new_account'];
		foreach ( $email_templates as $email ) {
			if(!in_array($email->id,$disabled)) {
				$emails[$email->id] = $email->get_title();
			}
		}

		return $emails;
	}

	//Get currency types
	public static function get_currency_codes() {
		$currency_code_options = get_woocommerce_currencies();
		foreach ( $currency_code_options as $code => $name ) {
			$currency_code_options[ $code ] = $name . ' (' . get_woocommerce_currency_symbol( $code ) . ')';
		}
		return $currency_code_options;
	}

	//Get error message for billingo api
	public function get_fetch_error_message($call = 'vat') {
		$message = '';
		if($call == 'vat' && count($this->get_vat_options()) == 1) {
			$message = '<span class="wc-billingo-plus-settings-error"><span class="dashicons dashicons-warning"></span> '.__('Nem sikerült lekérni az elérhető adókulcsokat. Ellenőrizd a publikus és privát kulcsokat!').'</span>';
		}
		if($call == 'blocks' && count($this->get_invoice_blocks()) == 0) {
			$message = '<span class="wc-billingo-plus-settings-error"><span class="dashicons dashicons-warning"></span> '.__('Nem sikerült lekérni az elérhető számlatömböket. Ellenőrizd a publikus és privát kulcsokat!').'</span>';
		}
		return $message;
	}

	public function get_vat_number_error_message() {
		$message = '';
		$surbma_hc_fields = get_option('surbma_hc_fields');
		if($surbma_hc_fields && $surbma_hc_fields['taxnumber']) {
			$message = '<span class="wc-billingo-plus-settings-error"><span class="dashicons dashicons-warning"></span> '.__('Az adószám mezőt a HuCommerce bővítmény már megjeleníti a pénztár oldalon').'</span>';
		}
		return $message;
	}

	public function generate_pro_html( $key, $data ) {
		$field    = $this->plugin_id . $this->id . '_' . $key;
		$defaults = array(
			'class'             => 'button-secondary',
			'css'               => '',
			'custom_attributes' => array(),
			'desc_tip'          => false,
			'description'       => '',
			'title'             => '',
		);
		$data = wp_parse_args( $data, $defaults );
		ob_start();
		?>
		<tr valign="top" class="wc-billingo-plus-section-pro <?php if(get_option('_wc_billingo_pro_enabled')): ?>wc-billingo-plus-section-pro-active<?php endif; ?>">
			<td>
				<div class="notice notice-error inline" style="display:none;"><p></p></div>
				<div class="wc-billingo-plus-section-pro-flex">
					<?php if(get_option('_wc_billingo_pro_enabled')): ?>
						<div class="wc-billingo-plus-section-pro-active">
							<strong><?php esc_html_e('Pro verzió aktiválva', 'wc-billingo-plus'); ?></strong>
							<small><?php echo esc_html(get_option('_wc_billingo_pro_key')); ?> / <?php echo esc_html(get_option('_wc_billingo_pro_email')); ?></small>
						</div>
						<a href="https://szamlazz.visztpeter.me/billingo-dokumentacio" target="_blank" class="button-primary"><?php esc_html_e('Support', 'wc-billingo-plus'); ?></a>
						<button class="button-secondary" type="button" name="<?php echo esc_attr( $field ); ?>-deactivate" id="<?php echo esc_attr( $field ); ?>-deactivate"><?php esc_html_e('Deaktiválás', 'wc-billingo-plus'); ?></button>
					<?php else: ?>
						<div class="wc-billingo-plus-section-pro-form">
							<h3><?php esc_html_e('WooCommerce Billingo PRO verzió', 'wc-billingo-plus'); ?></h3>
							<p><?php esc_html_e('Ha már megvásároltad, add meg a termékkulcsot és a vásárláshoz használt e-mail címet.', 'wc-billingo-plus'); ?></p>
							<?php echo $this->get_tooltip_html( $data ); ?>
							<fieldset>
								<input class="input-text regular-input" type="text" name="woocommerce_wc_billingo_pro_key" id="woocommerce_wc_billingo_pro_key" value="" placeholder="<?php esc_attr_e('Termékkulcs', 'wc-billingo-plus'); ?>"><br>
								<input class="input-text regular-input" type="text" name="woocommerce_wc_billingo_pro_email" id="woocommerce_wc_billingo_pro_email" value="" placeholder="<?php esc_attr_e('Vásárláshoz használt email cím', 'wc-billingo-plus'); ?>">
								<p><button class="button-primary" type="button" name="<?php echo esc_attr( $field ); ?>-submit" id="<?php echo esc_attr( $field ); ?>-submit"><?php esc_html_e('Aktiválás', 'wc-billingo-plus'); ?></button></p>
							</fieldset>
						</div>
						<div class="wc-billingo-plus-section-pro-cta">
							<h4><?php esc_html_e('Miért érdemes megvennem?', 'wc-billingo-plus'); ?></h4>
							<ul>
								<li><?php esc_html_e('Automata számlakészítés', 'wc-billingo-plus'); ?></li>
								<li><?php esc_html_e('Automata díjbekérő létrehozása', 'wc-billingo-plus'); ?></li>
								<li><?php esc_html_e('Automata sztornózás', 'wc-billingo-plus'); ?></li>
								<li><?php esc_html_e('Automata teljesítettnek jelölés', 'wc-billingo-plus'); ?></li>
								<li><?php esc_html_e('Prémium support és még sok más', 'wc-billingo-plus'); ?></li>
							</ul>
							<div class="wc-billingo-plus-section-pro-cta-button">
								<a href="https://szamlazz.visztpeter.me/billingo"><?php esc_html_e('PRO verzió vásárlása', 'wc-billingo-plus'); ?></a>
								<span>
									<small><?php esc_html_e('nettó', 'wc-billingo-plus'); ?></small>
									<strong><?php esc_html_e('10.000 Ft', 'wc-billingo-plus'); ?></strong>
								</span>
							</div>
						</div>
					<?php endif; ?>
				</div>
			</td>
		</tr>
	</tbody>
</table>
<table class="form-table">
	<tbody>
		<?php
		return ob_get_clean();
	}

	public function pro_check() {

		if ( !current_user_can( 'edit_shop_orders' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		}
		$pro_key = sanitize_text_field($_POST['key']);
		$pro_email = sanitize_email($_POST['email']);

		$args = array(
				'request' => 'activation',
				'email' => $pro_email,
				'licence_key' => $pro_key,
				'product_id' => 'WC_BILLINGO'
		);

		//Execute request (function below)
		$base_url = add_query_arg('wc-api', 'software-api', WC_Billingo_Plus_Settings::$activation_url);
		$target_url = $base_url . '&' . http_build_query( $args );
		$data = wp_remote_get( $target_url );
		$result = json_decode($data['body']);

		if(isset($result->activated) && $result->activated) {

			//Store the key and email
			update_option('_wc_billingo_pro_key', $pro_key);
			update_option('_wc_billingo_pro_email', $pro_email);
			update_option('_wc_billingo_pro_enabled', true);

			wp_send_json_success();

		} else {

			wp_send_json_error(array(
				'message' => esc_html__('Nem sikerült az aktiválás, kérlek ellenőrizd az adatok helyességét.', 'wc-billingo-plus')
			));

		}

	}

	public function pro_deactivate() {
		if ( !current_user_can( 'edit_shop_orders' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		}
		$pro_key = get_option('_wc_billingo_pro_key');
		$pro_email = get_option('_wc_billingo_pro_email');

		$args = array(
				'request' => 'activation_reset',
				'email' => $pro_email,
				'licence_key' => $pro_key,
				'product_id' => 'WC_BILLINGO'
		);

		//Execute request (function below)
		$base_url = add_query_arg('wc-api', 'software-api', WC_Billingo_Plus_Settings::$activation_url);
		$target_url = $base_url . '&' . http_build_query( $args );
		$data = wp_remote_get( $target_url );
		$result = json_decode($data['body']);

		if(isset($result->reset) && $result->reset) {

			//Store the key and email
			delete_option('_wc_billingo_pro_key');
			delete_option('_wc_billingo_pro_email');
			delete_option('_wc_billingo_pro_enabled');

			wp_send_json_success();

		} else {

			wp_send_json_error(array(
				'message' => esc_html__('Nem sikerült a deaktiválás, kérlek ellenőrizd az adatok helyességét.', 'wc-billingo-plus')
			));

		}

	}

	//Refresh blocks with ajax
	public function reload_blocks() {
		if ( !current_user_can( 'edit_shop_orders' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		}

		global $wc_billingo_plus;
		$blocks = $wc_billingo_plus->get_billingo_invoice_blocks(true);

		$blocks_array = array();
		foreach ($blocks as $block_id => $block_name) {
			$blocks_array[] = array('id' => $block_id, 'name' => $block_name);
		}

		wp_send_json_success($blocks_array);
	}

	public function generate_payment_methods_html($key, $data) {
		ob_start();

		global $wc_billingo_plus;
		$payment_methods = $this->get_payment_methods();
		$billingo_payment_methods = $wc_billingo_plus->get_billingo_payment_methods();
		$saved_values = get_option('wc_billingo_plus_payment_method_options');
		?>
		<tr valign="top">
			<th scope="row" class="titledesc"><?php esc_html_e( 'Fizetési módok:', 'woocommerce' ); ?></th>
			<td class="forminp">
				<div class="wc_input_table_wrapper">
					<?php if(empty($billingo_payment_methods)): ?>
						<span class="wc-billingo-plus-settings-error"><span class="dashicons dashicons-warning"></span> <?php esc_html_e('Nem sikerült lekérni az elérhető fizetési módokat. Ellenőrizd a publikus és privát kulcsokat!'); ?></span>
					<?php endif; ?>
					<table class="widefat wc_input_table wc-billingo-plus-payment-table" cellspacing="0">
						<thead>
							<tr>
								<th><?php esc_html_e( 'Fizetési mód', 'wc-billingo-plus' ); ?></th>
								<th><?php esc_html_e( 'Billingo azon.', 'wc-billingo-plus' ); ?></th>
								<th><?php esc_html_e( 'Fizetési határidő(nap)', 'wc-billingo-plus' ); ?></th>
								<th><?php esc_html_e( 'Teljesítettnek jelölés', 'wc-billingo-plus' ); ?> <?php if($data['disabled']): ?><i class="wc_billingo_pro_label">PRO</i><?php endif; ?></th>
								<th><?php esc_html_e( 'Díjbekérő létrehozása', 'wc-billingo-plus' ); ?> <?php if($data['disabled']): ?><i class="wc_billingo_pro_label">PRO</i><?php endif; ?></th>
							</tr>
						</thead>
						<tbody class="wc-billingo-plus-settings-inner-table">
							<?php foreach ( $payment_methods as $payment_method_id => $payment_method ): ?>
								<?php
								if($saved_values && isset($saved_values[esc_attr( $payment_method_id )])) {
									$value_billingo_id = esc_attr( $saved_values[esc_attr( $payment_method_id )]['billingo_id']);
									$value_deadline = esc_attr( $saved_values[esc_attr( $payment_method_id )]['deadline']);
									$value_complete = $saved_values[esc_attr( $payment_method_id )]['complete'];
									$value_request = $saved_values[esc_attr( $payment_method_id )]['proform'];
								} else {
									$value_billingo_id = '';
									$value_deadline = '';
									$value_complete = false;
									$value_request = false;
								}
								?>
								<tr>
									<td class="wc-billingo-plus-settings-inner-table-title"><strong><?php echo $payment_method; ?></strong></td>
									<td class="wc-billingo-plus-settings-inner-table-select">
										<?php
										woocommerce_form_field( 'wc_billingo_plus_payment_method_options['.esc_attr($payment_method_id).'][billingo_id]', array(
											'type'        => 'select',
											'options'     => $billingo_payment_methods,
										), $value_billingo_id );
										?>
									</td>
									<td class="wc-billingo-plus-settings-inner-table-field"><input type="number" name="wc_billingo_plus_payment_method_options[<?php echo esc_attr( $payment_method_id ); ?>][deadline]" value="<?php echo $value_deadline; ?>" /></td>
									<td class="wc-billingo-plus-settings-inner-table-cb"><input <?php disabled( $data['disabled'] ); ?> type="checkbox" name="wc_billingo_plus_payment_method_options[<?php echo esc_attr( $payment_method_id ); ?>][complete]" value="1" <?php checked( $value_complete ); ?> /></td>
									<td class="wc-billingo-plus-settings-inner-table-cb"><input <?php disabled( $data['disabled'] ); ?> type="checkbox" name="wc_billingo_plus_payment_method_options[<?php echo esc_attr( $payment_method_id ); ?>][proform]" value="1" <?php checked( $value_request ); ?> /></td>
								</tr>
							<?php endforeach ?>
						</tbody>
					</table>
				</div>
			</td>
		</tr>
		<?php
		return ob_get_clean();

	}

	public function generate_rounding_html($key, $data) {
		ob_start();

		$saved_values = get_option('wc_billingo_plus_rounding_options');
		if(empty($saved_values)) {
			$currency = get_woocommerce_currency();
			$saved_values = array($currency=>0);
		}
		?>
		<tr valign="top">
			<th scope="row" class="titledesc"><?php esc_html_e( 'Kerekítés:', 'woocommerce' ); ?></th>
			<td class="forminp">
				<table class="widefat wc_input_table wc-billingo-plus-rounding-table">
					<thead>
						<tr>
							<th><?php esc_html_e( 'Pénznem', 'wc-billingo-plus' ); ?></th>
							<th><?php esc_html_e( 'Kerekítés', 'wc-billingo-plus' ); ?></th>
							<th class="wc-billingo-plus-settings-inner-table-actions"></th>
						</tr>
					</thead>
					<tbody class="wc-billingo-plus-settings-inner-table">
						<?php foreach ($saved_values as $currency => $rounding): ?>
							<tr>
								<td class="wc-billingo-plus-rounding-table-currency wc-billingo-plus-settings-inner-table-select">
									<?php
									woocommerce_form_field( 'wc_billingo_plus_rounding_options[][currency]', array(
										'type'        => 'select',
										'options'     => $this->get_currency_codes(),
									), $currency );
									?>
								</td>
								<td class="wc-billingo-plus-rounding-table-rounding wc-billingo-plus-settings-inner-table-select">
									<?php
									woocommerce_form_field( 'wc_billingo_plus_rounding_options[][rounding]', array(
										'type'        => 'select',
										'options'     => array(
											0 => __('Nincs', 'wc-billingo'),
											1 => __('1-re', 'wc-billingo'),
											5 => __('5-re', 'wc-billingo'),
											10 => __('10-re', 'wc-billingo'),
										),
									), intval($rounding) );
									?>
								</td>
								<td class="wc-billingo-plus-settings-inner-table-actions">
									<a href="#" class="wc-billingo-plus-rounding-table-remove"><span class="dashicons dashicons-dismiss"></span></a>
									<a href="#" class="wc-billingo-plus-rounding-table-add"><span class="dashicons dashicons-plus-alt"></span></a>
								</td>
							</tr>
						<?php endforeach; ?>
					</tbody>
				</table>
				<p class="description"><?php esc_html_e( 'Ha több pénznemben is árulsz, beállíthatod mindegyikhez, hogy milyen kerekítés legyen', 'wc-billingo-plus' ); ?></p>
			</td>
		</tr>
		<?php
		return ob_get_clean();

	}

	public function save_payment_options() {

		//Save payment options
		$accounts = array();
		if ( isset( $_POST['wc_billingo_plus_payment_method_options'] ) ) {
			foreach ($_POST['wc_billingo_plus_payment_method_options'] as $payment_method_id => $payment_method) {
				$deadline = wc_clean($payment_method['deadline']);
				$complete = isset($payment_method['complete']) ? true : false;
				$proform = isset($payment_method['proform']) ? true : false;

				if(isset($payment_method['billingo_id'])) {
					$billingo_id = wc_clean($payment_method['billingo_id']);
				} else {
					$billingo_id = 0;
				}

				$accounts[$payment_method_id] = array(
					'billingo_id' => $billingo_id,
					'deadline' => $deadline,
					'complete' => $complete,
					'proform'  => $proform
				);
			}
		}
		update_option( 'wc_billingo_plus_payment_method_options', $accounts );

		//Save rounding options
		$roundings = array();
		if ( isset( $_POST['wc_billingo_plus_rounding_options'] ) ) {
			foreach ($_POST['wc_billingo_plus_rounding_options'] as $currency => $rounding) {
				$roundings[$rounding['currency']] = wc_clean($rounding['rounding']);
			}
		}
		update_option( 'wc_billingo_plus_rounding_options', $roundings );

	}

}

endif;
