<?php
/*
Plugin Name: Woo Billingo Plus
Plugin URI: http://visztpeter.me
Description: Nem hivatalos, de jobb Billingo összeköttetés WooCommercehez
Author: Viszt Péter
Version: 1.0.2
WC requires at least: 3.0.0
WC tested up to: 3.7.0
*/

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

//Load billingo API
require_once plugin_dir_path(__FILE__) . 'vendor/autoload.php';

//Generate stuff on plugin activation
function wc_billingo_plus_activate() {
	$upload_dir =  wp_upload_dir();

	$files = array(
		array(
			'base' 		=> $upload_dir['basedir'] . '/wc-billingo-plus',
			'file' 		=> 'index.html',
			'content' 	=> ''
		)
	);

	foreach ( $files as $file ) {
		if ( wp_mkdir_p( $file['base'] ) && ! file_exists( trailingslashit( $file['base'] ) . $file['file'] ) ) {
			if ( $file_handle = @fopen( trailingslashit( $file['base'] ) . $file['file'], 'w' ) ) {
				fwrite( $file_handle, $file['content'] );
				fclose( $file_handle );
			}
		}
	}
}
register_activation_hook( __FILE__, 'wc_billingo_plus_activate' );

//Setup Billingo API
use Billingo\API\Connector\HTTP\Request;

class WC_Billingo_Plus {

	public static $plugin_prefix;
	public static $plugin_url;
	public static $plugin_path;
	public static $plugin_basename;
	public static $version;
	protected static $background_generator = null;

  //Construct
	public function __construct() {

		//Default variables
		self::$plugin_prefix = 'wc_billingo_plus_';
		self::$plugin_basename = plugin_basename(__FILE__);
		self::$plugin_url = plugin_dir_url(self::$plugin_basename);
		self::$plugin_path = trailingslashit(dirname(__FILE__));
		self::$version = '1.0.2';

		//Plugin loaded
		add_action( 'plugins_loaded', array( $this, 'init' ) );

  }

	//Load plugin stuff
	public function init() {

		//Background invoice generator
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-background-generator.php' );
		self::$background_generator = new WC_Billingo_Plus_Background_Generator();

		// Load includes
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-settings.php' );
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-admin-notices.php' );
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-product-options.php' );

		//Health check for WP 5.2+
		global $wp_version;
		if ( version_compare( $wp_version, '5.2-alpha', 'ge' ) ) {
			require_once( plugin_dir_path( __FILE__ ) . 'includes/class-health-check.php' );
		}

		//Plugin links
		add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'plugin_action_links' ) );

		//Settings page
		add_filter( 'woocommerce_integrations', array( $this, 'add_integration' ) );

		//Admin functions
		add_action( 'admin_init', array( $this, 'admin_init' ) );
		add_action( 'add_meta_boxes', array( $this, 'metabox' ), 10, 2 );

		//Ajax functions
		add_action( 'wp_ajax_wc_billingo_plus_generate_invoice', array( $this, 'generate_invoice_with_ajax' ) );
		add_action( 'wp_ajax_wc_billingo_plus_mark_completed', array( $this, 'mark_completed_with_ajax' ) );
		add_action( 'wp_ajax_wc_billingo_plus_resend_email', array( $this, 'resend_email_with_ajax' ) );
		add_action( 'wp_ajax_wc_billingo_plus_void_invoice', array( $this, 'void_invoice_with_ajax' ) );
		add_action( 'wp_ajax_wc_billingo_plus_toggle_invoice', array( $this, 'toggle_invoice' ) );

		//Create a hook based on the status setup in settings to auto-generate invoice
		if(get_option('_wc_billingo_pro_enabled')) {
			$order_auto_invoice_status = str_replace( 'wc-', '', $this->get_option('auto_invoice_status', 'completed') );
			add_action( 'woocommerce_order_status_'.$order_auto_invoice_status, array( $this, 'on_order_complete' ) );
			add_action( 'woocommerce_checkout_order_processed', array( $this, 'on_order_processing' ) );

			$order_auto_void_status = str_replace( 'wc-', '', $this->get_option('auto_void_status', 'no') );
			if($order_auto_void_status != 'no') {
				add_action( 'woocommerce_order_status_'.$order_auto_void_status, array( $this, 'on_order_deleted' ) );
			}
		}

		//Order list button
		add_action( 'woocommerce_admin_order_actions_end', array( $this, 'add_listing_actions' ) );
		add_filter('woocommerce_my_account_my_orders_actions', array( $this, 'orders_download_button' ), 10,2);

		//VAT form
		if($this->get_option('vat_number_form') == 'yes') {
			add_filter( 'woocommerce_checkout_fields' , array( $this, 'add_vat_number_checkout_field' ) );
			add_filter( 'woocommerce_before_checkout_form' , array( $this, 'add_vat_number_info_notice' ) );
			add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'save_vat_number' ) );
			add_action( 'woocommerce_admin_order_data_after_billing_address', array( $this, 'display_vat_number' ) );

			if($this->get_option('vat_number_validate') == 'yes') {
				add_action( 'woocommerce_after_checkout_validation', array( $this, 'vat_number_validate' ), 10, 2);
				add_action( 'wp_enqueue_scripts', array( $this, 'frontend_js' ));
			}
		}

		//Attach invoice to emails
		if($this->get_option('email_attachment', 'no') == 'yes') {
			if($this->get_option('email_attachment_position', 'beginning') == 'beginning') {
				add_action('woocommerce_email_before_order_table', array( $this, 'email_attachment'), 10, 4);
				add_action('woocommerce_subscriptions_email_order_details', array( $this, 'email_attachment'), 10, 4);
			} else {
				add_action('woocommerce_email_customer_details', array( $this, 'email_attachment'), 30, 4);
			}
		}

		//Attach invoice to emails
		if($this->get_option('email_attachment_file', 'no') == 'yes') {
			add_filter( 'woocommerce_email_attachments', array( $this, 'email_attachment_file'), 10, 3 );
		}

	}

	//Integration page
	public function add_integration( $integrations ) {
		$integrations[] = 'WC_Billingo_Plus_Settings';
		return $integrations;
	}

  //Add CSS & JS
	public function admin_init() {
		wp_enqueue_script( 'wc_billingo_plus_admin_js', plugins_url( '/assets/js/admin.min.js',__FILE__ ), array('jquery'), WC_Billingo_Plus::$version, TRUE );
		wp_enqueue_style( 'wc_billingo_plus_admin_css', plugins_url( '/assets/css/admin.css',__FILE__ ), array(), WC_Billingo_Plus::$version );

		$wc_billingo_plus_local = array( 'loading' => plugins_url( '/assets/images/ajax-loader.gif',__FILE__ ) );
		wp_localize_script( 'wc_billingo_plus_admin_js', 'wc_billingo_plus_params', $wc_billingo_plus_local );
  }

	//Frontend JS
	public function frontend_js() {
		wp_enqueue_script( 'wc_billingo_plus_frontend_js', plugins_url( '/assets/js/frontend.min.js',__FILE__ ), array(), WC_Billingo_Plus::$version );
	}

	//Meta box on order page
	public function metabox( $post_type, $post ) {
		add_meta_box('wc_billingo_plus_metabox', 'Billingo Pro', array( $this, 'render_meta_box_content' ), 'shop_order', 'side');
	}

	//Render metabox content
	public function render_meta_box_content($post) {
		$order = wc_get_order($post->ID);
		include( dirname( __FILE__ ) . '/includes/views/html-metabox.php' );
	}

	//Generate Invoice with Ajax
	public function generate_invoice_with_ajax() {
		check_ajax_referer( 'wc_billingo_generate_invoice', 'nonce' );
		$order_id = intval($_POST['order']);
		$response = $this->generate_invoice($order_id);
		wp_send_json_success($response);
	}

	//Mark completed with Ajax
	public function mark_completed_with_ajax() {
		check_ajax_referer( 'wc_billingo_generate_invoice', 'nonce' );
		$order_id = intval($_POST['order']);
		$response = $this->generate_invoice_complete($order_id);
		wp_send_json_success($response);
	}

	//Resend email notification with Ajax
	public function resend_email_with_ajax() {
		check_ajax_referer( 'wc_billingo_generate_invoice', 'nonce' );
		$order_id = intval($_POST['order']);
		$response = $this->resend_email($order_id);
		wp_send_json_success($response);
	}

	//Cancel Invoice with Ajax
	public function void_invoice_with_ajax() {
		check_ajax_referer( 'wc_billingo_generate_invoice', 'nonce' );
		$order_id = intval($_POST['order']);
		$response = $this->generate_void_invoice($order_id);
		wp_send_json_success($response);
	}

	//Generate XML for Szamla Agent
	public function generate_invoice($orderId, $proform = false) {
		$order = wc_get_order($orderId);
		$order_items = $order->get_items();

		//Response
		$response = array();
		$response['error'] = false;
		$response['type'] = 'invoice';

		//Load billingo API
		$billingo = $this->get_billingo_api();

		//Create client
		$clientData = [
			'name' => '',
			'email' => $order->get_billing_email(),
			'phone' => $order->get_billing_phone(),
			'taxcode' => '',
			'billing_address' => [
				'street_name' => $order->get_billing_address_1(),
				'city' => $order->get_billing_city(),
				'postcode' => $order->get_billing_postcode(),
				'country' => WC()->countries->countries[$order->get_billing_country()] ? : '',
			]
		];

		//Set client name
		if($this->get_option('nev_csere') == 'yes') {
			$clientData['name'] = $order->get_billing_last_name().' '.$order->get_billing_first_name();
		} else {
			$clientData['name'] = $order->get_formatted_billing_full_name();
		}

		//Set company name
		if($order->get_billing_company()) {
			if($this->get_option('company_name')) {
				$clientData['name'] = $order->get_billing_company().' - '.$clientData['name'];
			} else {
				$clientData['name'] = $order->get_billing_company();
			}
		}

		//TAX number
		if($taxcode = $order->get_meta( 'billingo_plus_adoszam' )) {
			$clientData['taxcode'] = $taxcode;
		}

		//HuCommerce
		if($taxcode = $order->get_meta( '_billing_tax_number' )) {
			$clientData['taxcode'] = $taxcode;
		}

		//EU VAT Assistant
		if($taxcode = $order->get_meta( 'vat_number' )) {
			$clientData['taxcode'] = $taxcode;
		}

		//Save client
		try {
			$this->log_debug_messages($clientData, 'create-client-'.$orderId);
			$client = $billingo->post('clients', apply_filters('wc_billingo_plus_client', $clientData, $order));
			if(!$client['id']) {
				$response['error'] = true;
				$response['messages'][] = esc_html__('Nem sikerült létrehozni az ügyfelet.', 'billingo');
				return $response;
			}
		} catch (Exception $e) {
			$response['error'] = true;
			$response['messages'][] = $e->getMessage();
			$this->log_error_messages($e, 'create_client-'.$orderId);
			$order->add_order_note(esc_html__('Sikertelen számlakészítés. Nem sikerült az ügyfelet létrehozni. Hibakód: ', 'wc-billingo') . $e->getMessage());
			return $response;
		}

		//If custom details
		if(isset($_POST['note']) && isset($_POST['deadline']) && isset($_POST['completed'])) {
			$note = sanitize_text_field($_POST['note']);
			$deadline = intval($_POST['deadline']);
			$completed_date = sanitize_text_field($_POST['completed']);
		} else {
			$note = $this->get_option('note');
			$deadline = $this->get_payment_method_deadline($order->get_payment_method());
			$completed_date = date('Y-m-d');
		}

		//Replace customer email and phone number in note
		$note_replacements = array('{customer_email}' => $order->get_billing_email(), '{customer_phone}' => $order->get_billing_phone());
		$note = str_replace( array_keys( $note_replacements ), array_values( $note_replacements ), $note);

		//Create invoce data array
		$invoiceData = [
			'fulfillment_date' => $completed_date,
			'due_date' => ($deadline) ? date('Y-m-d', strtotime('+'.$deadline.' days')) : date('Y-m-d'),
			'payment_method' => (int)$this->check_payment_method_options($order->get_payment_method(), 'billingo_id'),
			'comment' => $note,
			'template_lang_code' => $this->get_option('language', 'hu'),
			'electronic_invoice' => (int)$this->get_invoice_type($order),
			'currency' => $order->get_currency() ?: 'HUF',
			'client_uid' => $client['id'],
			'block_uid' => (int)$this->get_option('block_uid'),
			'round_to' => $this->get_rounding_option($order),
			'type' => 3
		];

		//If its a proform invoice
		if($proform || (isset($_POST['type']) && $_POST['type'] == 'proform')) {
			$invoiceData['type'] = 1;
			$response['type'] = 'proform';
			$proform = true;
		}

		//Check if we need to mark it as completed based on the payment method
		if($this->check_payment_method_options($order->get_payment_method(), 'complete')) {
			$invoiceData['is_paid'] = true;
		} else {
			$invoiceData['is_paid'] = false;
		}

		//Language based on WPML
		if($this->get_option('language_wpml') == 'yes') {
			$wpml_lang_code = $order->get_meta('wpml_language');
			if(!$wpml_lang_code && function_exists('pll_get_post_language')){
				$wpml_lang_code = pll_get_post_language($orderId, 'locale');
			}
			if($wpml_lang_code && in_array($wpml_lang_code, array('hu', 'de', 'en', 'it', 'fr', 'hr', 'ro', 'sk'))) {
				$invoiceData['template_lang_code'] = $wpml_lang_code;
			}
		}

		//Add products
		foreach( $order_items as $order_item ) {

			$product_item = array(
				'description' => $order_item->get_name(),
				'qty' => $order_item->get_quantity(),
				'unit' => $this->get_option('unit_type', __('db', 'wc-billingo-plus')),
				'vat_id' => $this->get_order_item_tax_id($order, $order_item),
				'net_unit_price' => 0,
			);

			//Custom product name
			if($order_item->get_product() && $order_item->get_product()->get_meta('wc_billingo_plus_tetel_nev') && $order_item->get_product()->get_meta('wc_billingo_plus_tetel_nev') != '') {
				$product_item['description'] = $order_item->get_product()->get_meta('wc_billingo_plus_tetel_nev');
			}

			//Custom unit type
			if($order_item->get_product() && $order_item->get_product()->get_meta('wc_billingo_plus_mennyisegi_egyseg') && $order_item->get_product()->get_meta('wc_billingo_plus_mennyisegi_egyseg') != '') {
				$product_item['unit'] = $order_item->get_product()->get_meta('wc_billingo_plus_mennyisegi_egyseg');
			}

			//Set price
			if($this->get_option('separate_coupon') == 'yes') {
				if (round($order_item->get_subtotal(), wc_get_price_decimals()) != 0) {
					$product_item['net_unit_price'] = round($order_item->get_subtotal(),2)/$order_item->get_quantity();
				}
			} else {
				if (round($order_item->get_total(), wc_get_price_decimals()) != 0) {
					$product_item['net_unit_price'] = round($order_item->get_total(),2)/$order_item->get_quantity();
				}
			}

			//Item note
			$note = '';

			//Show variation details if needed
			$product_name = $order_item->get_name();
			if(strpos($product_name, ' - ') === false) {
				$note = strip_tags(wc_display_item_meta( $order_item, array(
					'before'    => "\n- ",
					'separator' => "\n- ",
					'after'     => "",
					'echo'      => false,
					'autop'     => false,
				)));
				if($note != '') $note .= "\n";
			}

			//Custom note
			if($order_item->get_product() && $order_item->get_product()->get_meta('wc_billingo_plus_megjegyzes')) {
				$note .= $order_item->get_product()->get_meta('wc_billingo_plus_megjegyzes');
			}

			//If we need to show sale price in the note
			if($this->get_option('discount_note') && $order_item->get_product() && $order_item->get_product()->is_on_sale()) {
				$sale_price = $order_item->get_product()->get_sale_price();
				$net_unit_price = $order_item->get_total()/$order_item->get_quantity();
				$gross_unit_price = ($order_item->get_total()+$order_item->get_total_tax())/$order_item->get_quantity();

				if(round($sale_price,2) == round($net_unit_price,2) || round($sale_price,2) == round($gross_unit_price,2)) {
					if(get_option( 'woocommerce_prices_include_tax') == 'no') {
						$afakulcs = 1+$order_item->get_total_tax()/$order_item->get_total();
					} else {
						$afakulcs = 1;
					}
					$regular_price = $order_item->get_product()->get_regular_price()*$afakulcs;
					$original_price = $regular_price*$order_item->get_quantity();
					$applied_sale = $original_price-($order_item->get_total()+$order_item->get_total_tax());
					$discounted_price = $order_item->get_total()+$order_item->get_total_tax();
					$discount_note = $this->get_option('discount_note');
					$discount_note_replacements = array('{eredeti_ar}' => wc_price($original_price), '{kedvezmeny_merteke}' => wc_price($applied_sale), '{kedvezmenyes_ar}' => wc_price($discounted_price));
					$discount_note = str_replace( array_keys( $discount_note_replacements ), array_values( $discount_note_replacements ), $discount_note);
					$discount_note = strip_tags($discount_note);
					$discount_note = html_entity_decode($discount_note);
					$note .= $discount_note;
				}
			}

			//Set note
			$product_item['item_comment'] = $note;

			//Append to items
			$invoiceData['items'][] = $product_item;

		}

		//Shipping
		foreach( $order->get_items( 'shipping' ) as $item_id => $shipping_item_obj ){
			$order_shipping = $shipping_item_obj->get_total();

			//Skip if free shipping is hidden
			if($this->get_option('hide_free_shipping') == 'yes' && $order_shipping == 0) {
				continue;
			}

			$product_item = array(
					'description' => $shipping_item_obj->get_method_title(),
					'qty' => 1,
					'unit' => $this->get_option('unit_type', __('db', 'wc-billingo-plus')),
					'vat_id' => $this->get_order_shipping_tax_id($order, $shipping_item_obj),
					'net_unit_price' => round($order_shipping,2)
			);

			//Append to items
			$invoiceData['items'][] = $product_item;

		}

		//Fees
		$fees = $order->get_fees();
		if(!empty($fees)) {
			foreach( $fees as $fee ) {
				$product_item = array(
						'description' => $fee['name'],
						'qty' => 1,
						'unit' => $this->get_option('unit_type', __('db', 'wc-billingo-plus')),
						'net_unit_price' => round($fee["line_total"],2),
						'vat_id' => $this->get_order_shipping_tax_id($order, $fee)
				);

				$invoiceData['items'][] = $product_item;
			}
		}

		//Discount
		if ( $order->get_discount_total() > 0 ) {
			$discout_details = $this->get_coupon_invoice_item_details($order);
			//If coupon is a separate item
			if($this->get_option('separate_coupon') == 'yes') {
				$product_item = array(
					'description' => $discout_details['description'],
					'item_comment' => $discout_details["item_comment"],
					'qty' => 1,
					'unit' => $this->get_option('unit_type', __('db', 'wc-billingo-plus')),
					'net_unit_price' => -round($order->get_total_discount(),2),
					'vat_id' => $invoiceData['items'][0]['vat_id']
				);
				$invoiceData['items'][] = $product_item;
			} else {
				//Add space if theres already something in the comment
				if($invoiceData['comment']) {
					$invoiceData['comment'] .= ' ';
				}
				$invoiceData['comment'] .= $discout_details["item_comment"];
			}
		}

		//Create invoice
		try {
			$this->log_debug_messages($invoiceData, 'generate-invoice-'.$orderId);
			$invoice = $billingo->post('invoices', apply_filters('wc_billingo_plus_invoice', $invoiceData, $order));
			if (!$invoice['id']) {
				$response['error'] = true;
				$response['messages'][] = esc_html__('Nem sikerült létrehozni a számlát.', 'wc-billingo-plus');
				return $response;
			}
		} catch (Exception $e) {

			//Create response
			$response['error'] = true;
			$response['messages'][] = esc_html__('Sikertelen számlakészítés.', 'wc-billingo-plus');
			$response['messages'][] = $e->getMessage();
			$order->add_order_note(esc_html__('Sikertelen számlakészítés. Hibakód: ', 'wc-billingo-plus') . $e->getMessage());

			//Callbacks
			$this->log_error_messages($e, 'generate_invoice-'.$orderId);
			do_action('wc_billingo_plus_after_invoice_error', $order, $e);
			return $response;
		}

		//Send via email if needed
		$auto_email_sent = false;
		if($this->get_option('auto_email')) {
			try {
				$billingo->get("invoices/{$invoice['id']}/send");
				$auto_email_sent = true;
			} catch (Exception $e) {
				$this->log_error_messages($e, 'send_email-'.$orderId);
				$response['messages'][] = esc_html__('Nem sikerült elküldeni emailben a számlát', 'wc-billingo-plus');
				return false;
			}
		}

		//Save file if needed
		if($this->get_option('download_invoice')) {
			try {
				$pdf_file = $this->get_pdf_file_path($response['type'], $orderId);
				$email = $billingo->downloadInvoice($invoice['id'], $pdf_file['path']);
				$pdf_file_name = $pdf_file['name'];
			} catch (Exception $e) {
				$this->log_error_messages($e, 'download_invoice-'.$orderId);
				$response['messages'][] = esc_html__('Nem sikerült letölteni a számlát.', 'wc-billingo-plus');
				$response['messages'][] = $e->getMessage();
			}
		} else {
			//Create download link
			try {
				$pdf_file_name = $billingo->get("invoices/{$invoice['id']}/code");
				$pdf_file_name = $pdf_file_name['code'];
			} catch (Exception $e) {
				$this->log_error_messages($e, 'create_download_link-'.$orderId);
				$response['messages'][] = esc_html__('Nem sikerült létrehozni a letöltési linket a számlához.', 'wc-billingo-plus');
			}
		}

		//Get invoice data
		$invoice_name = $invoice['attributes']['invoice_no'];
		$invoice_id = $invoice['id'];
		$invoice_pdf = $pdf_file_name;

		//Create response
		$response['name'] = $invoice_name;

		//Save order meta and comment
		if($proform) {
			$response['messages'][] = ($auto_email_sent) ? esc_html__('Díjbekérő sikeresen létrehozva és elküldve a vásárlónak emailben.','wc-billingo-plus') : esc_html__('Díjbekérő sikeresen létrehozva.','wc-billingo-plus');

			//Update order notes
			$order->add_order_note(esc_html__('Billingo díjbekérő sikeresen létrehozva. A számla sorszáma: ', 'wc-billingo') . $invoice_name);

			//Store the filename
			$order->update_meta_data( '_wc_billingo_plus_proform_id', $invoice_id );
			$order->update_meta_data( '_wc_billingo_plus_proform_name', $invoice_name );
			$order->update_meta_data( '_wc_billingo_plus_proform_pdf', $invoice_pdf );

			//Return download links
			$response['link'] = $this->generate_download_link($order, 'proform');

		} else {
			$response['messages'][] = ($auto_email_sent) ? esc_html__('Számla sikeresen létrehozva és elküldve a vásárlónak emailben.','wc-billingo-plus') : esc_html__('Számla sikeresen létrehozva.','wc-billingo-plus');

			//Update order notes
			$order->add_order_note(esc_html__('Billingo számla sikeresen létrehozva. A számla sorszáma: ', 'wc-billingo') . $invoice_name);

			//Store the filename
			$order->update_meta_data( '_wc_billingo_plus_invoice_id', $invoice_id );
			$order->update_meta_data( '_wc_billingo_plus_invoice_name', $invoice_name );
			$order->update_meta_data( '_wc_billingo_plus_invoice_pdf', $invoice_pdf );

			//Mark as paid if needed
			if($invoiceData['is_paid']) {
				$order->update_meta_data( '_wc_billingo_plus_completed', time() );
			}

			//Return download links
			$response['link'] = $this->generate_download_link($order);

		}

		//Delete void invoice if exists
		$order->delete_meta_data( '_wc_billingo_plus_void_id' );
		$order->delete_meta_data( '_wc_billingo_plus_void_name' );
		$order->delete_meta_data( '_wc_billingo_plus_void_pdf' );

		//Save the order
		$order->save();

		//Run action on successful invoice creation
		do_action('wc_billingo_plus_after_invoice_success', $order, $response);

		return $response;
	}

	//Autogenerate invoice
	public function on_order_complete( $order_id ) {

		//Only generate invoice, if it wasn't already generated & only if automatic invoice is enabled
		if($this->get_option('auto_generate') == 'yes' && !$this->is_invoice_generated($order_id)) {

			//Check if we generate this invoice deferred
			if($this->get_option('defer') == 'yes') {
				self::$background_generator->push_to_queue(
					array(
						'invoice_type' => 'final',
						'order_id' => $order_id
					)
				);

				self::$background_generator->save()->dispatch();
			} else {
				$return_info = $this->generate_invoice($order_id);

				//If there was an error while generating invoices automatically
				if($return_info && $return_info['error']) {
					$this->on_auto_invoice_error($order_id);
				}
			}

		}

	}

	//Autogenerate proform invoice
	public function on_order_processing( $order_id ) {

		//Only generate invoice, if it wasn't already generated & only if automatic invoice is enabled
		$order = new WC_Order($order_id);
		$payment_method = $order->get_payment_method();

		if($this->check_payment_method_options($payment_method, 'proform') && !$this->is_invoice_generated($order_id)) {
			if($this->get_option('defer') == 'yes') {
				self::$background_generator->push_to_queue(
					array(
						'invoice_type' => 'proform',
						'order_id' => $order_id
					)
				);
				self::$background_generator->save()->dispatch();
			} else {
				$return_info = $this->generate_invoice($order_id,true);
			}
		}

	}

	//Autogenerate invoice
	public function on_order_deleted( $order_id ) {

		//Only generate sztornó, if regular invoice already generated & only if automatic invoice is enabled
		if($this->get_option('auto_generate') == 'yes' && $this->is_invoice_generated($order_id)) {
			$response = $this->generate_void_invoice($order_id);
		}

	}

	//Send email on error
	public function on_auto_invoice_error( $order_id ) {
		update_option('_wc_billingo_plus_error', $order_id);

		//Check if we need to send an email todo
		if($this->get_option('error_email')) {
			$order = wc_get_order($order_id);
			$mailer = WC()->mailer();
			$content = wc_get_template_html( 'includes/emails/invoice-error.php', array(
				'order'         => $order,
				'email_heading' => esc_html__('Sikertelen számlakészítés', 'wc-billingo-plus'),
				'plain_text'    => false,
				'email'         => $mailer,
				'sent_to_admin' => true,
			), '', plugin_dir_path( __FILE__ ) );
			$recipient = $this->get_option('error_email');
			$subject = esc_html__("Sikertelen számlakészítés", 'wc-billingo-plus');
			$headers = "Content-Type: text/html\r\n";
			$mailer->send( $recipient, $subject, $content, $headers );
		}

	}

	//Check if it was already generated or not
	public function is_invoice_generated( $order_id ) {
		$order = wc_get_order($order_id);
		$invoice_id = $order->get_meta('_wc_billingo_plus_invoice_id');
		$invoice_own = $order->get_meta('_wc_billingo_plus_own');
		return ($invoice_id || $invoice_own);
	}

	//Add icon to order list to show invoice
	public function add_listing_actions( $order ) {
		$order_id = $order->get_id();

		if($this->is_invoice_generated($order_id)):
		?>
			<a href="<?php echo $this->generate_download_link($order); ?>" class="button tips wc-action-button wc-billingo-plus-order-button" target="_blank" alt="" data-tip="<?php esc_html_e('Billingo számla','wc-billingo-plus'); ?>">
				<img src="<?php echo WC_Billingo_Plus::$plugin_url . 'assets/images/icon-invoice.svg'; ?>" alt="" width="12" height="16">
			</a>
		<?php
		endif;

		if($order->get_meta('_wc_billingo_plus_proform_pdf')):
		?>
			<a href="<?php echo $this->generate_download_link($order,'proform'); ?>" class="button tips wc-action-button wc-billingo-plus-order-button" target="_blank" alt="" data-tip="<?php esc_html_e('Billingo díjbekérő','wc-billingo-plus'); ?>">
				<img src="<?php echo WC_Billingo_Plus::$plugin_url . 'assets/images/icon-proform.svg'; ?>" alt="" width="12" height="16">
			</a>
		<?php
		endif;
	}

	//Generate download url
	public function generate_download_link( $order, $type = false, $absolute = false) {
		if($order) {
			$pdf_name = '';
			if($type && $type == 'proform') {
				$pdf_name = $order->get_meta('_wc_billingo_plus_proform_pdf');
			} else if($type && $type == 'void') {
				$pdf_name = $order->get_meta('_wc_billingo_plus_void_pdf');
			} else {
				$pdf_name = $order->get_meta('_wc_billingo_plus_invoice_pdf');
			}

			if($pdf_name) {
				if(strpos($pdf_name, '.pdf') !== false) {
					$UploadDir = wp_upload_dir();
					$UploadURL = $UploadDir['baseurl'];
					if($absolute) {
						$pdf_file_url = $UploadDir['basedir'].'/wc-billingo-plus/'.$pdf_name;
					} else {
						$pdf_file_url = $UploadURL.'/wc-billingo-plus/'.$pdf_name;
					}
					return apply_filters('wc_billingo_plus_download_link', esc_url($pdf_file_url), $order);
				} else {
					return apply_filters('wc_billingo_plus_download_link', esc_url('https://www.billingo.hu/access/c:' . $pdf_name), $order);
				}
			} else {
				return false;
			}
		} else {
			return false;
		}
	}

	//If the invoice is already generated without the plugin
	public function toggle_invoice() {
		check_ajax_referer( 'wc_billingo_generate_invoice', 'nonce' );

		if ( !current_user_can( 'edit_shop_orders' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		}
		$orderid = intval($_POST['order']);
		$order = wc_get_order($orderid);
		$note = sanitize_text_field($_POST['note']);
		$invoice_own = $order->get_meta('_wc_billingo_plus_own');
		$response = array();

		if($invoice_own) {
			$response['state'] = 'on';
			$order->delete_meta_data('_wc_billingo_plus_own');
			$response['messages'][] = esc_html__('Számlakészítés visszakapcsolva.','wc-billingo-plus');
		} else {
			$response['state'] = 'off';
			$order->update_meta_data( '_wc_billingo_plus_own', $note );
			$response['messages'][] = esc_html__('Számlakészítés kikapcsolva.','wc-billingo-plus');
		}

		//Save the order
		$order-save();

		wp_send_json_success($response);
	}

	//Add vat number field to checkout page
	public function add_vat_number_checkout_field($fields) {
		$show = true;

		if($this->get_option('vat_number_form_min') == 'yes' && WC()->cart->get_taxes_total() < 100000) {
			$show = false;
		}

		if($show) {
			$fields['billing']['billingo_plus_adoszam'] = apply_filters('wc_billingo_plus_tax_number_field', array(
				 'label'     => esc_html__('Adószám', 'wc-billingo-plus'),
				 'placeholder'   => _x('12345678-1-23', 'placeholder', 'wc-billingo-plus'),
				 'required'  => false,
				 'class'     => array('form-row-wide'),
				 'clear'     => true,
				 'priority'	 => 35
			));
		}

		return $fields;
	}

	//Create VAT notification on checkout
	public function add_vat_number_info_notice($checkout) {
		if(WC()->cart->get_taxes_total() > 100000) {
			wc_print_notice( $this->get_option('vat_number_notice'), 'notice' );
		}
	}

	//Save vat number to order meta
	public function save_vat_number( $order_id ) {
		if ( ! empty( $_POST['billingo_plus_adoszam'] ) ) {
			$order = wc_get_order( $order_id );
			$order->update_meta_data( 'billingo_plus_adoszam', sanitize_text_field( $_POST['billingo_plus_adoszam'] ) );
			$order->save();
		}
	}

	//Show vat number on order details page
	public function display_vat_number($order){
		$order_id = $this->get_order_id($order);
		if($adoszam = $order->get_meta('billingo_plus_adoszam')) {
			echo '<p><strong>'.esc_html__('Adószám', 'wc-billingo-plus').':</strong> ' . $adoszam . '</p>';
		}
	}

	//Validate the vat number format
	public function vat_number_validate($fields, $errors) {
		if($fields['billingo_plus_adoszam']) {

			//Validate general format
			if(!preg_match('/^(\d{7})(\d)\-([1-5])\-(0[2-9]|[13][0-9]|2[02-9]|4[0-4]|51)$/', sanitize_text_field($fields['billingo_plus_adoszam']))) {
				$errors->add( 'validation', apply_filters('wc_billingo_plus_tax_validation_message', esc_html__( 'Az adószám formátuma nem megfelelő.', 'wc-billingo-plus'), $fields) );
			}

		}
	}

	//Mark invoice as payed
	public function generate_invoice_complete($orderId) {
		$order = new WC_Order($orderId);

		//Response
		$response = array();
		$response['error'] = false;

		//Load billingo API
		$billingo = $this->get_billingo_api();

		//Get existing invoice data
		$invoice_id = $order->get_meta('_wc_billingo_plus_invoice_id');
		$completed = $order->get_meta('_wc_billingo_plus_completed');

		//If invoice doesn't exists
		if(!$invoice_id) {
			$response['error'] = true;
			$response['messages'][] = esc_html__('Nem található a számla.', 'billingo');
			return $response;
		}

		//If already marked as payed
		if($completed) {
			$response['error'] = true;
			$response['messages'][] = esc_html__('A számla már fizetettnek lett jelölve.', 'billingo');
			return $response;
		}

		//Try to mark invoice payed
		try {
			$paydata = array(
				'date' => date('Y-m-d'),
				'amount' => round($order->get_total(),2),
				'payment_method' => (int)$this->check_payment_method_options($order->get_payment_method(), 'billingo_id')
			);

			$paid = $billingo->post("invoices/{$invoice_id}/pay", apply_filters('wc_billingo_plus_complete', $paydata, $order));

			if(!$paid['fully_paid']) {
				$response['error'] = true;
				$response['messages'][] = esc_html__('Nem sikerült fizetettnek jelölni a számlát.', 'billingo');
				return $response;
			}

		} catch (Exception $e) {
			$response['error'] = true;
			$response['messages'][] = esc_html__('Nem sikerült fizetettnek jelölni a számlát.', 'wc-billingo-plus');
			$response['messages'][] = $e->getMessage();
			$this->log_error_messages($e, 'generate_invoice_complete-'.$orderId);
			$order->add_order_note( esc_html__( 'Nem sikerült fizetettnek jelölni a számlát! Hibakód: ', 'wc-billingo-plus' ) . $e->getMessage());
			return $response;
		}

		//Store as a custom field
		$order->update_meta_data( '_wc_billingo_plus_completed', time() );

		//Update order notes
		$order->add_order_note( esc_html__( 'Számla sikeresen fizetettnek jelölve', 'wc-billingo-plus' ) );

		//Save order
		$order-save();

		//Response
		$response['completed'] = date('Y-m-d');

		return $response;
	}

	//Generate void invoice
	public function generate_void_invoice($orderId) {
		$order = new WC_Order($orderId);

		//Response
		$response = array();
		$response['error'] = false;

		//Load billingo API
		$billingo = $this->get_billingo_api();

		//Get existing invoice id
		$invoice_id = $order->get_meta('_wc_billingo_plus_invoice_id');

		if(!$invoice_id) {
			$response['error'] = true;
			$response['messages'][] = esc_html__('Nem található a számla.', 'billingo');
			return $response;
		}

		//Create void invoice
		try {
			$invoice_void = $billingo->get("invoices/{$invoice_id}/cancel");
			if(!$invoice_void['id']) {
				$response['error'] = true;
				$response['messages'][] = esc_html__('Nem sikerült létrehozni a sztornó számlát.', 'billingo');
				return $response;
			}
		} catch (Exception $e) {
			$response['error'] = true;
			$response['messages'][] = esc_html__('Nem sikerült létrehozni a sztornó számlát.', 'wc-billingo-plus');
			$response['messages'][] = $e->getMessage();
			$this->log_error_messages($e, 'generate_void_invoice-'.$orderId);
			$order->add_order_note( esc_html__( 'Sztornó számlakészítés sikertelen! Hibakód: ', 'wc-billingo-plus' ) . $e->getMessage());
			return $response;
		}

		//Create download link
		try {
			$pdf_file_name = $billingo->get("invoices/{$invoice_void['id']}/code");
			$pdf_file_name = $pdf_file_name['code'];
		} catch (Exception $e) {
			$this->log_error_messages($e, 'create_download_link-'.$orderId);
			$response['messages'][] = esc_html__('Nem sikerült létrehozni a letöltési linket a sztornó számlához.', 'wc-billingo-plus');
		}

		//Get invoice data
		$invoice_void_name = $invoice_void['attributes']['invoice_no'];
		$invoice_void_id = $invoice_void['id'];
		$invoice_void_pdf = $pdf_file_name;

		//Create response
		$response['name'] = $invoice_void_name;

		//Save void meta
		$order->update_meta_data( '_wc_billingo_plus_void_id', $invoice_void_id );
		$order->update_meta_data( '_wc_billingo_plus_void_name', $invoice_void_name );
		$order->update_meta_data( '_wc_billingo_plus_void_pdf', $invoice_void_pdf );

		//Delete existing meta
		$order->delete_meta_data( '_wc_billingo_plus_invoice_id' );
		$order->delete_meta_data( '_wc_billingo_plus_invoice_name' );
		$order->delete_meta_data( '_wc_billingo_plus_invoice_pdf' );
		$order->delete_meta_data( '_wc_billingo_plus_proform_id' );
		$order->delete_meta_data( '_wc_billingo_plus_proform_name' );
		$order->delete_meta_data( '_wc_billingo_plus_proform_pdf' );
		$order->delete_meta_data( '_wc_billingp_plus_completed' );

		//Update order notes
		$response['messages'][] = esc_html__('Sztornó számla létrehozva.','wc-billingo-plus');
		$order->add_order_note(esc_html__('Sztornó számla sikeresen létrehozva. A számla sorszáma: ', 'wc-billingo') . $invoice_void_name);

		//Return download links
		$response['link'] = $this->generate_download_link($order, 'void');

		//Save the order
		$order->save();

		return $response;
	}

	//Resend email notification
	public function resend_email($orderId) {
		$order = new WC_Order($orderId);

		//Response
		$response = array();
		$response['error'] = false;

		//Load billingo API
		$billingo = $this->get_billingo_api();

		//Get existing invoice data
		$invoice_id = $order->get_meta('_wc_billingo_plus_invoice_id');

		//If invoice doesn't exists
		if(!$invoice_id) {
			$response['error'] = true;
			$response['messages'][] = esc_html__('Nem található a számla.', 'billingo');
			return $response;
		}

		//Try to send out the invoice
		try {
			$email = $billingo->get("invoices/{$invoice_id}/send");
		} catch (Exception $e) {
			$this->log_error_messages($e, 'resend_email-'.$orderId);
			$response['error'] = true;
			$response['messages'][] = esc_html__('Nem sikerült elküldeni emailben a számlát.', 'wc-billingo-plus');
			$response['messages'][] = $e->getMessage();
			return $response;
		}

		return $response;
	}

	//Add download icons to order details page
	public function orders_download_button($actions, $order) {
		$order_id = $order->get_id();
		if($this->get_option('customer_download','no') == 'yes') {

			//Add invoice link
			if($order->get_meta('_wc_billingo_plus_invoice_pdf')) {
				$link = $this->generate_download_link($order);
				$actions['billingo_plus_pdf'] = array(
					'url'  => $link,
					'name' => esc_html__( 'Számla', 'wc-billing-plus' )
				);
			}

			//Add payment request link
			if($order->get_meta('_wc_billingo_plus_proform_pdf')) {
				$link_request = $this->generate_download_link($order,'proform');
				$actions['billingo_plus_pdf'] = array(
					'url'  => $link_request,
					'name' => esc_html__( 'Díjbekérő', 'wc-billing-plus' )
				);
			}

			//Add payment request link
			if($order->get_meta('_wc_billingo_plus_void_pdf')) {
				$link_request = $this->generate_download_link($order,'void');
				$actions['billingo_plus_pdf'] = array(
					'url'  => $link_request,
					'name' => esc_html__( 'Sztornó számla', 'wc-billing-plus' )
				);
			}
		}
		return $actions;
	}

	//Get options stored
	public function get_option($key, $default = '') {
		$settings = get_option( 'woocommerce_wc_billingo_plus_settings', null );
		$value = $default;

		if($settings && isset($settings[$key])) {
			$value = $settings[$key];
		} else if(get_option($key)) {
			$value = get_option($key);
		}

		//Try to get password from wp-config
		if($key == 'private_key' && defined( 'WC_BILLINGO_PRIVATE_KEY' )) {
			$value = WC_BILLINGO_PRIVATE_KEY;
		}

		return apply_filters('wc_billingo_plus_get_option', $value, $key);
	}

	//Email attachment
	public function email_attachment($order, $sent_to_admin, $plain_text, $email){
		$order_id = $order->get_id();
		$invoices = array();

		$invoice_types = array('invoice', 'proform', 'void');
		foreach ($invoice_types as $invoice_type) {
			$invoice_email_ids = $this->get_option('email_attachment_'.$invoice_type, array());
			if($invoice_email_ids && !empty($invoice_email_ids)) {
				if(in_array($email->id,$invoice_email_ids)) {
					$invoices[$invoice_type] = array();
					$invoices[$invoice_type]['type'] = $invoice_type;
					$invoices[$invoice_type]['name'] = $order->get_meta('_wc_billingo_plus_'.$invoice_type.'_name');
					$invoices[$invoice_type]['link'] = $this->generate_download_link($order, $invoice_type);
				}
			}
		}

		if(!empty($invoices)) {
			if($plain_text) {
				wc_get_template( 'emails/plain/email-billingo-section.php', array( 'order' => $order, 'billingo_invoices' => $invoices ), '', plugin_dir_path( __FILE__ ) . 'includes/' );
			} else {
				wc_get_template( 'emails/email-billingo-section.php', array( 'order' => $order, 'billingo_invoices' => $invoices ), '', plugin_dir_path( __FILE__ ) . 'includes/' );
			}
		}
	}

	//Email attachment file
	public function email_attachment_file($attachments, $email_id, $order){
		$order_id = $order->get_id();

		$invoice_types = array('invoice', 'proform', 'void');
		foreach ($invoice_types as $invoice_type) {
			$invoice_email_ids = $this->get_option('email_attachment_'.$invoice_type, array());
			if($invoice_email_ids && !empty($invoice_email_ids)) {
				if(in_array($email_id,$invoice_email_ids)) {
					$pdf_name = $order->get_meta('_wc_billingo_plus_'.$invoice_type.'_pdf');
					if(strpos($pdf_name, '.pdf') !== false) {
						$attachments[] = $this->generate_download_link($order, $invoice_type, true);
					}
				}
			}
		}
		return $attachments;
	}

	//Plugin links
	public function plugin_action_links( $links ) {
		$action_links = array(
			'settings' => '<a href="' . esc_url(admin_url( 'admin.php?page=wc-settings&tab=integration&section=wc_billingo_plus' )) . '" aria-label="' . esc_attr__( 'Billingo Pro Beállítások', 'wc-billingo-plus' ) . '">' . esc_html__( 'Beállítások', 'wc-billingo-plus' ) . '</a>',
			'documentation' => '<a href="https://szamlazz.visztpeter.me/billingo-dokumentacio/" target="_blank" aria-label="' . esc_attr__( 'Billingo Pro Dokumentáció', 'wc-billingo-plus' ) . '">' . esc_html__( 'Dokumentáció', 'wc-billingo-plus' ) . '</a>'
		);

		if (!get_option('_wc_billingo_pro_enabled') ) {
			$action_links['get-pro'] = '<a target="_blank" rel="noopener noreferrer" style="color:#46b450;" href="https://szamlazz.visztpeter.me/billingo" aria-label="' . esc_attr__( 'Billingo Pro verzió', 'wc-billingo-plus' ) . '">' . esc_html__( 'Pro verzió', 'wc-billingo-plus' ) . '</a>';
		}
		return array_merge( $action_links, $links );
	}

	public function get_invoice_type($order) {
		if($order->get_billing_company()) {
			$type = $this->get_option('invoice_type_company', 'paper');
		} else {
			$type = $this->get_option('invoice_type', 'paper');
		}
		return ($type == 'electronic');
	}

	public function get_rounding_option($order) {
		$order_currency = $order->get_currency();
		$rounding_type = 0;
		$rounding_options = $this->get_option('wc_billingo_plus_rounding_options', array());
		foreach ($rounding_options as $currency => $rounding) {
			if($currency == $order_currency) {
				$rounding_type = intval($rounding);
			}
		}
		return $rounding_type;
	}

	public function check_payment_method_options($payment_method_id, $option) {
		$found = false;
		$payment_method_options = $this->get_option('wc_billingo_plus_payment_method_options');
		if(isset($payment_method_options[$payment_method_id]) && isset($payment_method_options[$payment_method_id][$option])) {
			$found = $payment_method_options[$payment_method_id][$option];
		}
		return $found;
	}

	public function get_payment_method_deadline($payment_method_id) {
		$deadline = $this->get_option('payment_deadline');
		$payment_method_options = $this->get_option('wc_billingo_plus_payment_method_options');
		if($payment_method_options && isset($payment_method_options[$payment_method_id]) && isset($payment_method_options[$payment_method_id]['deadline'])) {
			$deadline = $payment_method_options[$payment_method_id]['deadline'];
		}
		return $deadline;
	}

	public function get_order_item_tax_id($order, $item) {

		//If a fixed value is set in settings
		if($this->get_option('afakulcs') != '') {
			return (int)$this->get_option('afakulcs');
		}

		$tax_item_label = '';

		//Check what is set in woocommerce tax settings
		if(wc_tax_enabled()) {
			$tax_items_labels = array();
			$valid_tax_labels = array('AM', 'EU', 'FAD', 'ÁTHK', 'AAM', 'AKK', 'TAM', 'EUK', 'MAA');

			//Get all tax labels indexed by rate id
			foreach ( $order->get_items('tax') as $tax_item ) {
				$tax_items_labels[$tax_item->get_rate_id()] = $tax_item->get_label();
			}

			//Get line item tax id and find label
			if(count($tax_items_labels) > 0) {
				$taxes = $item->get_taxes();
				foreach( $taxes['subtotal'] as $rate_id => $tax ){
					$tax_item_label = $tax_items_labels[$rate_id];
				}
			}

			//If its not a valid label
			if(!in_array($tax_item_label, $valid_tax_labels)) {
				$tax_item_label = '';
			}
		}

		//If theres no ID, return percentage value
		if($tax_item_label == '') {
			if($this->get_option('separate_coupon') == 'yes') {
				$tax_item_label = round( ($item->get_subtotal_tax()/$item->get_subtotal()) * 100 );
			} else {
				$tax_item_label = round( ($item->get_total_tax()/$item->get_total()) * 100 );
			}
		}

		//Get Billingo vat ids
		$vat_id = 1;
		$billingo_vat_ids = $this->get_billingo_vat_ids();
		foreach ($billingo_vat_ids as $billingo_vat_id => $billingo_vat_label) {
			if($tax_item_label == str_replace('%', '', $billingo_vat_label)) {
				$vat_id = $billingo_vat_id;
			}
		}

		return (int)$vat_id;
	}

	public function get_order_shipping_tax_id($order, $shipping_item_obj) {
		$tax_item_label = '';

		//If a fixed value is set in settings
		if($this->get_option('afakulcs') != '') {
			return (int)$this->get_option('afakulcs');
		}

		//Check what is set in woocommerce tax settings
		if(wc_tax_enabled()) {
			$valid_tax_labels = array('AM', 'EU', 'FAD', 'ÁTHK', 'AAM', 'AKK', 'TAM', 'EUK', 'MAA');

			$tax_data = $shipping_item_obj->get_taxes();
			foreach ( $order->get_items('tax') as $tax_item ) {
				$tax_item_id    = $tax_item->get_rate_id();
				$tax_item_total = isset( $tax_data['total'][ $tax_item_id ] ) ? $tax_data['total'][ $tax_item_id ] : '';

				if($tax_item_total != '') {
					$tax_item_label = $tax_item->get_label();
				}
			}

			//If its not a valid label
			if(!in_array($tax_item_label, $valid_tax_labels)) {
				$tax_item_label = '';
			}
		}

		if($tax_item_label == '') {
			$order_shipping = $shipping_item_obj->get_total();
			$order_shipping_tax = $shipping_item_obj->get_total_tax();
			$tax_item_label = round(($order_shipping_tax/$order_shipping)*100);
		}

		//Get Billingo vat ids
		$vat_id = 1;
		$billingo_vat_ids = $this->get_billingo_vat_ids();
		foreach ($billingo_vat_ids as $billingo_vat_id => $billingo_vat_label) {
			if($tax_item_label == str_replace('%', '', $billingo_vat_label)) {
				$vat_id = $billingo_vat_id;
			}
		}

		return (int)$vat_id;
	}

	public function get_coupon_invoice_item_details($order) {
		$details = array(
			"description" => esc_html__('Kedvezmény', 'wc-billingo-plus'),
			"item_comment" => ''
		);

		$order_discount = method_exists( $order, 'get_discount_total' ) ? $order->get_discount_total() : $order->order_discount;
		if ( $order_discount > 0 ) {
			$coupons = implode(', ', $order->get_used_coupons());
			$discount = strip_tags(html_entity_decode($order->get_discount_to_display()));
			$details["description"] = sprintf( esc_html__( '%1$s kedvezmény a következő kupon kóddal: %2$s', 'wc-billingo-plus' ), $discount, $coupons );

			if($this->get_option('separate_coupon_name')) {
				$details["description"] = $this->get_option('separate_coupon_name');
			}

			if($this->get_option('separate_coupon_desc')) {
				$discount_note_replacements = array('{kedvezmeny_merteke}' => $discount, '{kupon}' => $coupons);
				$discount_note = str_replace( array_keys( $discount_note_replacements ), array_values( $discount_note_replacements ), $this->get_option('separate_coupon_desc'));
				$details["item_comment"] = $discount_note;
			}
		}

		return $details;
	}

	//Start a new billingo api request
	public function get_billingo_api() {
		if(isset($_POST['woocommerce_wc_billingo_plus_public_key'])) {
			$billingo = new Request(array(
				'public_key' => esc_attr($_POST['woocommerce_wc_billingo_plus_public_key']),
				'private_key' => esc_attr($_POST['woocommerce_wc_billingo_plus_private_key'])
			));
		} else {
			$billingo = new Request(array(
				'public_key' => $this->get_option('public_key'),
				'private_key' => $this->get_option('private_key')
			));
		}
		return $billingo;
	}

	//Get billingo VAT id's
	public function get_billingo_vat_id($percentage) {
		$vat_ids = $this->get_billingo_vat_ids();

		//Find the closest one
		$vat_id = 1;
		foreach ($vat_ids as $billingo_vat_id => $billingo_vat_id_name) {
			if ($billingo_vat_id_name == $percentage) {
				$vat_id = $billingo_vat_id;
			}
		}

		return (int)$vat_id;
	}

	//Query for the VAT ids
	public function get_billingo_vat_ids($skip_numeric = false) {
		$vat_ids = get_transient('wc_billingo_plus_vat_ids');
		if (!$vat_ids) {

			//Load billingo API
			$billingo = $this->get_billingo_api();
			$billingo_vat_ids = false;

			try {
				$billingo_vat_ids = $billingo->get('vat');
			} catch (Exception $e) {
				$this->log_error_messages($e, 'get_vat_ids');
			}

			//Create a simple array
			$vat_ids = array();

			if (is_array($billingo_vat_ids)) {
				foreach ($billingo_vat_ids as $billingo_vat_id) {
					$vat_ids[$billingo_vat_id['id']] = $billingo_vat_id['attributes']['description'];
				}

				//Save vat ids for a day
				set_transient('wc_billingo_plus_vat_ids', $vat_ids, 60 * 60 * 24);
			}
		}

		if ($skip_numeric) {
			foreach ($vat_ids as $k => $v) {
				if (strpos($v, '%') !== false) {
					unset($vat_ids[$k]);
				}
			}
		}
		return $vat_ids;
	}

	//Get payment method ids
	public function get_billingo_payment_methods() {
		$payment_methods = get_transient('wc_billingo_plus_payment_methods');
		if (!$payment_methods) {

			//Load billingo API
			$billingo = $this->get_billingo_api();
			$billingo_payment_methods = false;

			try {
				$billingo_payment_methods = $billingo->get('payment_methods/hu');
			} catch (Exception $e) {
				$this->log_error_messages($e, 'get_payment_methods');
			}

			//Create a simple array
			$payment_methods = array();

			if(is_array($billingo_payment_methods)) {
				foreach ($billingo_payment_methods as $payment_method) {
					$payment_methods[$payment_method['id']] = $payment_method['attributes']['name'];
				}

				//Save payment methods for a day
				set_transient('wc_billingo_plus_payment_methods', $payment_methods, 60 * 60 * 24);
			}
		}

		return $payment_methods;
	}

	public function get_billingo_invoice_blocks($refresh = false) {
		$invoice_blocks = get_transient('wc_billingo_plus_invoice_blocks');
		if (!$invoice_blocks || $refresh) {

			//Load billingo API
			$billingo = $this->get_billingo_api();
			$billingo_invoice_blocks = false;

			try {
				$billingo_invoice_blocks = $billingo->get('invoices/blocks');
			} catch (Exception $e) {
				$this->log_error_messages($e, 'get_invoice_blocks');
			}

			//Create a simple array
			$invoice_blocks = array();

			if(is_array($billingo_invoice_blocks)) {
				foreach ($billingo_invoice_blocks as $billingo_invoice_block) {
					$invoice_blocks[$billingo_invoice_block['id']] = $billingo_invoice_block['attributes']['name'];
				}

				//Save vat ids for a day
			}
			set_transient('wc_billingo_plus_invoice_blocks', $invoice_blocks, 60 * 60 * 24);
		}

		return $invoice_blocks;
	}

	//Log error message if needed
	public function log_error_messages($e, $source) {
		$logger = wc_get_logger();
		$error = $e->getMessage();
		if(method_exists($e, 'getResponse')) {
			$error = $e->getResponse()->getBody()->getContents();
		}
		$logger->error(
			$source.' - '.$error,
			array( 'source' => 'wc_billingo_plus' )
		);
	}

	//Log debug messages if needed
	public function log_debug_messages($data, $source) {
		if($this->get_option('debug', 'no') == 'yes') {
			$logger = wc_get_logger();
			$logger->debug(
				$source.' - '.json_encode($data),
				array( 'source' => 'wc_billingo_plus' )
			);
		}
	}

	//Get file path for pdf files
	public function get_pdf_file_path($type, $order_id) {
		$upload_dir = wp_upload_dir( null, false );
		$location = $upload_dir['basedir'] . '/wc-billingo-plus/';
		$random_file_name = substr(md5(rand()),5);
		$pdf_file_name = implode( '-', array( $type, $order_id, $random_file_name ) ).'.pdf';
		$pdf_file_path = $location.'/'.$pdf_file_name;
		return array('name' => $pdf_file_name, 'path' => $pdf_file_path);
	}


}

//WC Detection
if ( ! function_exists( 'is_woocommerce_active' ) ) {
	function is_woocommerce_active() {
		$active_plugins = (array) get_option( 'active_plugins', array() );

		if ( is_multisite() ) {
			$active_plugins = array_merge( $active_plugins, get_site_option( 'active_sitewide_plugins', array() ) );
		}

		return in_array( 'woocommerce/woocommerce.php', $active_plugins ) || array_key_exists( 'woocommerce/woocommerce.php', $active_plugins ) ;
	}
}


//WooCommerce inactive notice.
function wc_billingo_plus_woocommerce_inactive_notice() {
	if ( current_user_can( 'activate_plugins' ) ) {
		echo '<div id="message" class="error"><p>';
		printf( __( 'A %1$sWooCommerce Billingo Plus inaktív%2$s. A %3$sWooCommerce bővítménynek %4$s aktívnak kell lennie. %5$sTelepítsd és kapcsold be WooCommerce-et &raquo;%6$s', 'wc-billingo-plus' ), '<strong>', '</strong>', '<a href="http://wordpress.org/extend/plugins/woocommerce/">', '</a>', '<a href="' . esc_url( admin_url( 'plugins.php' ) ) . '">', '</a>' );
		echo '</p></div>';
	}
}

//Initialize
if ( is_woocommerce_active() ) {
	$GLOBALS['wc_billingo_plus'] = new WC_Billingo_Plus();
} else {
	add_action( 'admin_notices', 'wc_billingo_plus_woocommerce_inactive_notice' );
}
